/*
 * KDE Composite Manager, 
 * Copyright (C) 2003-2005 Fredrik Höglund <fredrik@kde.org>
 *
 * Based on xcompmgr,
 * Copyright (C) 2003 Keith Packard <keithp@keithp.com>
 *
 * Internal structure and naming style inspired by KWin,
 * Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 * Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Fredrik Höglund not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Fredrik Höglund makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * FREDRIK HÖGLUND DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL FREDRIK HÖGLUND BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _SHADOWPAINTER_H
#define _SHADOWPAINTER_H

#include <X11/Xlib.h>
#include <X11/extensions/Xrender.h>
#include <X11/extensions/Xfixes.h>

#undef Cursor
#undef CursorShape
#undef Shape

#include <fixx11h.h>
#include <QCache>
#include <QRect>

#include "globaldefs.h"

enum Tile { TopLeft=0, Top, TopRight, Right,
			BottomRight, Bottom, BottomLeft, Left, Center, NTiles };
		
class TileSet
{
	public:
		TileSet( double radius );
		~TileSet();

		Picture tile( Tile t ) const { return mTiles[t]; }
		Picture operator[]( Tile t ) const { return mTiles[t]; }
		int size() const { return mSize; }

	private:
		double gaussian( double x, double y );
		unsigned char sum( double *kernel, int x, int y );
		Picture createPicture( Picture source, int x, int y,
							   int width, int height, uint flags = NoFlags );

		Picture mTiles[NTiles];
		double mRadius;
		int mSize;
};


class ShadowPainter
{
	public:
		static ShadowPainter *self();
		~ShadowPainter();

		// Draws a shadow in destination with the specified geometry, radius and opacity.
		static void drawShadow( Picture destination, const QRect &r, double radius,
								double opacity = 1.0 )
		{
			self()->draw( destination, r, radius, opacity );
		}

		// Draws shadow pictures created by createShadowPicture().
		// The opacity parameter is relative to the opacity set with setShadowOpacity().
		static void drawShadowPicture( Picture destination, const QRect &rect,
									   Picture shadow, double opacity = 1.0 )
		{
			self()->draw( destination, rect, shadow, opacity );
		}

		// Creates a shadow picture of the specified size and with the specified radius.
		static Picture createShadowPicture( const QSize &size, double radius )
		{
			return self()->createShadow( size, radius );
		}

		// Calculates the shadow geometry for a given window geometry
		static QRect shadowGeometry( const QRect &r, double radius )
		{
			return self()->geometry( r, radius );
		}

	private:
		ShadowPainter();

		void draw( Picture destination, const QRect &r, double radius, double opacity = 1.0 );
		void draw( Picture destination, const QRect &r, Picture shadow, double opacity = 1.0 );
		Picture createShadow( const QSize &size, double radius );
		QRect geometry( const QRect &r, double radius );
		TileSet *findTileSet( double radius );

		void setupForOpacity( double opacity );

		QCache<int, TileSet> *mCache;
		Picture mSource;
		double  mLastOpacity;

		static ShadowPainter *sSelf;
};

#endif
