/*
 *   Copyright 2008 Aaron Seigo <aseigo@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2 or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef LDBBACKEND_H
#define LDBBACKEND_H

#include <KConfigBackend>

/**
 * This class implements a KConfigBackend using LDB
 */
class LDBBackend : public KConfigBackend
{
    Q_OBJECT

    public:
        LDBBackend(QObject* parent, const QVariantList& args);
        ~LDBBackend();

        ParseInfo parseConfig(const QByteArray& locale,
                              KEntryMap& pWriteBackMap,
                              ParseOptions options);

        bool writeConfig(const QByteArray& locale, KEntryMap& entryMap,
                         WriteOptions options, const KComponentData &data);

        bool isWritable() const;
        KConfigBase::AccessMode accessMode() const;
        void createEnclosing();
        void setFilePath(const QString& path);
        bool lock(const KComponentData& componentData);
        virtual void unlock();
        virtual bool isLocked() const;
};

K_EXPORT_KCONFIGBACKEND(ldb, LDBBackend)

#endif
