/**
 *  Copyright (C) 2005 Benjamin C Meyer (ben at meyerhome dot net)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Steet, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#include <QStringList>
#include <QTime>

#define LOOP 9999999

/**
 * TEST: QStringList walking, i.e. QList walking
 */
int main( int argc, char *argv[] ) {
    {
    QStringList list;
    for(uint i=0; i < LOOP; i++)
	list.append(QString("%1").arg(i));

    QTime t;
    t.start();
    //Indexing:
    for (int i = 0; i < list.size(); ++i)
        list[i];
        qDebug("index[]:  %d", t.elapsed());
    }
    
    {
    QStringList list;
    for(uint i=0; i < LOOP; i++)
	list.append(QString("%1").arg(i));

    QTime t;
    t.start();
    //Indexing:
    for (int i = 0; i < list.size(); ++i)
        list.at(i);
    qDebug("index at:  %d", t.elapsed());
    }
    
    {
    QStringList list;
    for(uint i=0; i < LOOP; i++)
	list.append(QString("%1").arg(i));

    QTime t;
    t.start();
    //Java-style iterator:
    QStringListIterator i(list);
    while (i.hasNext())
        i.next();
    qDebug("whilenext:  %d", t.elapsed());
    }

    {
    QStringList list;
    for(uint i=0; i < LOOP; i++)
	list.append(QString("%1").arg(i));

    QTime t;
    t.start();
    //STL-style iterator:
    QStringList::const_iterator i;
    const QStringList::const_iterator cend = list.constEnd();
    for (i = list.constBegin(); i != list.constEnd(); ++i)
        (*i);
    qDebug("stl:  %d", t.elapsed());
    }

    {
    QStringList list;
    for(uint i=0; i < LOOP; i++)
	list.append(QString("%1").arg(i));

    QTime t;
    t.start();
    //STL-style iterator:
    QStringList::const_iterator i;
    const QStringList::const_iterator cend = list.constEnd();
    for (i = list.constBegin(); i != cend; ++i)
        (*i);
    qDebug("stl:  %d", t.elapsed());
    }

    return 0;
}

