/*
    This file is part of Fate.

    Copyright (c) 2005 SUSE LINUX Products GmbH
    
    Author: Cornelius Schumacher <cschum@suse.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef KUPDATE_H
#define KUPDATE_H

#include <kupdateiface.h>

#include <kdialogbase.h>
#include <kactioncollection.h>

#include <qlabel.h>

class QTextView;
class QPushButton;
class KMainWindow;
class KProcess;

class AppStarter : public QObject
{
    Q_OBJECT
  public:
    AppStarter( const QCString &appname );
    
  signals:
    void appStarted( AppStarter * );
    
  protected slots:
    void slotApplicationRegistered( const QCString & );
    void destroyProcess( KProcess * );
    void emitAppStarted();

  private:
    QCString mAppname;
};

class StatusBarLabel : public QLabel
{
    Q_OBJECT
  public:
    enum State { Uptodate, UpdateAvailable, NoKinstaller, Connecting,
      Undefined };
  
    StatusBarLabel();

    void setState( State );

  signals:
    void clicked();

  protected:
    void mousePressEvent( QMouseEvent * );

  private:
    QColor mBgColor;
    QColor mFgColor;
};

class KUpdate : public QObject, public KUpdateIface
{
    Q_OBJECT
  public:
    KUpdate( KMainWindow *, const QString &appname );

    void initAction( KMainWindow *, KActionCollection * );
    void init();

    QString appname();
    QString currentVersion();
    QString newestVersion();

    void update();

    bool updateAvailable();

    void restart();

    QString errorMessage();

  public slots:
    void showDialog();
    void checkForUpdate();

  protected slots:
    void destroyAppStarter( AppStarter * ); 
    void slotRestart();

  private:
    KMainWindow *mMainWindow;
    QString mAppname;
    
    QString mCurrentVersion;
    QString mNewestVersion;  

    bool mUpdateAvailable;

    StatusBarLabel *mStatusLabel;

    QString mErrorMessage;
};

class KUpdateDialog : public KDialogBase
{
    Q_OBJECT
  public:
    KUpdateDialog( QWidget *parent, KUpdate * );

  protected slots:
    void update();
    
  private:
    QTextView *mView;
    QPushButton *mUpdateButton;

    KUpdate *mUpdate;
};

#endif
