/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mergethread.h"

#include "changelog.h"
#include "syncfile.h"

#include <KDebug>

Nepomuk::MergeThread::MergeThread( QObject* parent )
    : QThread( parent )
{
    start();
}


Nepomuk::MergeThread::~MergeThread()
{
}


void Nepomuk::MergeThread::stop()
{
    m_stopped = true;
    m_queueWaiter.wakeAll();
}


int Nepomuk::MergeThread::process(const Nepomuk::Sync::ChangeLog& changeLog )
{
    kDebug();
    m_queueMutex.lock();

    kDebug() << "Received ChangeLog -- " << changeLog.size();
    SyncRequest * request = new SyncRequest( changeLog );
    m_queue.enqueue( request );

    m_queueMutex.unlock();
    m_queueWaiter.wakeAll();

    return request->id();
}


void Nepomuk::MergeThread::run()
{
    m_stopped = false;

    while( !m_stopped ) {

        // lock for initial iteration
        m_queueMutex.lock();

        while( !m_queue.isEmpty() ) {

            SyncRequest* request = m_queue.dequeue();
            kDebug() << "Processing request #" << request->id() << " with size " << request->size();

            // unlock after queue utilization
            m_queueMutex.unlock();

            connect( request, SIGNAL(completed(int)),
                     this, SIGNAL(completed(int)) );
            connect( request, SIGNAL(multipleMerge(QString,QString)),
                     this, SIGNAL(multipleMerge(QString,QString)) );

            request->load();
            request->merge();

            if( !request->done() ) {
                QMutexLocker lock( &m_processMutex );
                m_processes[ request->id() ] = request;
            }

            disconnect( request, SIGNAL(completed(int)),
                        this, SIGNAL(completed(int)) );
            disconnect( request, SIGNAL(multipleMerge(QString,QString)),
                     this, SIGNAL(multipleMerge(QString,QString)) );

            if( request->done() ){
                m_processes.remove( request->id() );
                delete request;
            }

            m_queueMutex.lock();
        }

        kDebug() << "Waiting...";
        m_queueWaiter.wait( &m_queueMutex );
        m_queueMutex.unlock();
        kDebug() << "Woke up.";
    }
}


#include "mergethread.moc"
