/* This file is part of the KDE Project
   Copyright (c) 2009 Daniel Winter <dw@danielwinter.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILESYSTEM_H
#define FILESYSTEM_H

#include <QDateTime>
#include <QExplicitlySharedDataPointer>

#include <Nepomuk/Resource>

namespace Nepomuk
{

    class FilesystemData : public QSharedData
    {
    public:
        FilesystemData() : isValid(false) {}
       FilesystemData(const FilesystemData &other)
             : QSharedData(other), isValid(other.isValid), identifer(other.identifer), mountpoint(other.mountpoint), label(other.label)
                 , resource(other.resource), mounted(other.mounted), byteSize(other.byteSize), priority(other.priority)
                 , lastmounted(other.lastmounted) {
       }
       ~FilesystemData() {}

        bool isValid;
        QUrl identifer;
        QString mountpoint;
        QString label;
        Resource resource;
        bool mounted;
        qulonglong byteSize;
        int priority;
        QDateTime lastmounted;

    };


    class Filesystem
    {

    public:
        Filesystem() { d = new FilesystemData; }
        Filesystem( const QUrl& identifier, const QString& mountpoint, const QString& label, bool mounted, qulonglong byteSize, int priority = 5 );

        Filesystem(const Filesystem &other);
        ~Filesystem() { }

        void setMounted( bool mounted , const QString& mountpoint = QString() );

        QString mountpoint() const;
        int priority() const;
        int isValid() const { return d->isValid; }
        bool writeToNepomuk();


    private:
        void initNepomukResource();

    private:
        QExplicitlySharedDataPointer<FilesystemData> d;

    };

}
#endif // FILESYSTEM_H
