/* This file is part of the KDE Project
   Copyright (c) 2009 Daniel Winter <dw@danielwinter.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KMOUNTPOINT_HANDLER_H
#define KMOUNTPOINT_HANDLER_H

#include <QObject>

#include <KDebug>
#include <kmountpoint.h>

#include "abstractfilesystemhandler.h"
#include "filesystem.h"



class QTimer;

namespace Nepomuk
{

class KMountPointHandler : public AbstractFilesystemHandler
{
    Q_OBJECT

public:
    KMountPointHandler( QObject *parent, int pollingInterval, int defaultPriority, int maxPriority );
    virtual QList<Filesystem> mountedFilesystems() const;

private:
    QHash<QString, Filesystem> m_mountedFilesystems;
    QTimer *m_timer;
    int m_pollingInterval;
    int m_defaultPriority;
    int m_maxPriority;
    bool m_valid;
    QStringList m_fsTypeIgnoreList;

private:
    Filesystem filesystemForKMountPoint( KMountPoint::Ptr mp ) const;
    void update( bool emitSignals );
    bool shouldIgnoreMountPoint( KMountPoint::Ptr mp ) const;

private Q_SLOTS:
    void slotUpdate();

Q_SIGNALS:
    void filesystemMounted( Filesystem f );
    void filesystemUnmounted( Filesystem f );
};

}
#endif // KMOUNTPOINT_HANDLER_H
