/* This file is part of the KDE Project
   Copyright (c) 2009 Daniel Winter <dw@danielwinter.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SOLID_FILESYSTEM_HANDLER_H
#define SOLID_FILESYSTEM_HANDLER_H

#include <QObject>

#include <KDebug>

#include "abstractfilesystemhandler.h"
#include "filesystem.h"


namespace Solid {
    class Device;
    class StorageAccess;
    class StorageVolume;
}

namespace Nepomuk
{

class SolidFilesystemHandler : public AbstractFilesystemHandler
{
    Q_OBJECT

public:
    SolidFilesystemHandler( QObject *parent, int priority );
    virtual QList<Filesystem> mountedFilesystems() const;

private:
    class Entry {
        SolidFilesystemHandler* q;

        public:
           Entry() {}
           Entry( SolidFilesystemHandler* );

           QString m_udi;
           Filesystem m_filesystem;

           const Solid::StorageAccess* m_storageAccess;
           const Solid::StorageVolume* m_storageVolume;
    };

    QHash<QString, Entry> m_metadataCache;
    QHash<QString, Filesystem> m_mountedFilesystems;
    int m_priority;


    Entry* createCacheEntry( const Solid::Device& dev );
    void initCacheEntries();
    const Filesystem filesystemForEntry( const Entry& udi ) const;

private Q_SLOTS:
    void slotSolidDeviceAdded( const QString& udi );
    void slotSolidDeviceRemoved( const QString& udi );

    void slotAccessibilityChanged( bool accessible, const QString& udi );

Q_SIGNALS:
    void filesystemMounted( Filesystem f );
    void filesystemUnmounted( Filesystem f );
};

}
#endif // SOLID_FILESYTSEM_HANDLER_H
