/*
 * Copyright (c) 2010 Ivan Cukic <ivan.cukic(at)kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "activitymanager.h"

#include <QTimer>

#include <KConfig>
#include <KConfigGroup>

#include <Nepomuk/Resource>

#ifndef ACTIVITYMANAGER_P_H
#define ACTIVITYMANAGER_P_H

class ActivityManager::Private: public QObject {
    Q_OBJECT
public:
    Private();

    // Contains the IDs of all activities
    QStringList availableActivities;

    // Contains the IDs of running activities
    QStringList runningActivities;

    // URIs and WIDs for open resources
    QHash < WId, QSet < QUrl > > resources;
    QHash < QUrl, QSet < QString > > activities;
    QHash < WId, QString > applications;

    // Current activity
    QString currentActivity;

    // Configuration
    QTimer configSyncTimer;
    KConfig config;

public:
    void initConifg();

    KConfigGroup activitiesConfig();
    KConfigGroup mainConfig();
    QString activityName(const QString & id);

    Nepomuk::Resource activityResource(const QString & id) const;

public Q_SLOTS:
    void scheduleConfigSync();
    void configSync();

};

#endif // ACTIVITYMANAGER_P_H

