/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "contactresourceeditplugin.h"

#include "../src/propertyeditwidget.h"
#include "../src/nepomukdataaccess.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPalette>
#include <QColor>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QFormLayout>
#include <QSet>
#include <QUrl>
#include <QLabel>
#include <KComboBox>
#include <QMap>

#include <Nepomuk/Types/Property>
#include <Nepomuk/Types/Class>
#include <Nepomuk/Variant>
#include <Nepomuk/Resource>
#include <Soprano/Vocabulary/RDFS>
#include <nco.h>
#include <nie.h>

#include <KPluginFactory>
#include <KPluginLoader>
#include <KDebug>
#include <KLocalizedString>
#include <KAction>
#include <KMessageBox>

K_PLUGIN_FACTORY(ContactResourceEditPluginFactory, registerPlugin<Nepomuk::ContactResourceEditPlugin>();)
K_EXPORT_PLUGIN(ContactResourceEditPluginFactory("nepomuk_contactresourceeditplugin"))

class Nepomuk::ContactResourceEditPlugin::Private
{
public:
    Private() {
    }

    QList<PropertyEditWidget*> propertyEditWidgets;
    QLabel *photoLabel;
};

Nepomuk::ContactResourceEditPlugin::ContactResourceEditPlugin(QWidget *parent, const QList<QVariant>& ) :
        AbstractResourceEditPlugin(parent),
        d( new Private() )
{
}

Nepomuk::ContactResourceEditPlugin::~ContactResourceEditPlugin()
{
    delete d;
}

void Nepomuk::ContactResourceEditPlugin::initGui()
{
    if (layout()) {
        layout()->deleteLater();
    }

    //add main layout
    QVBoxLayout *mainLayout = new QVBoxLayout();
    mainLayout->setContentsMargins(0, 0, 0, 0);

    //add property editor widgets (and their layout)

    //form layout for info
    QFormLayout *formLayout = new QFormLayout;
    formLayout->setContentsMargins(1, 1, 1, 1);
    mainLayout->addLayout(formLayout);

    QList<QUrl> properties;
    properties << Nepomuk::Vocabulary::NCO::hasEmailAddress()
               << Nepomuk::Vocabulary::NCO::fullname()
               << Nepomuk::Vocabulary::NCO::phoneNumber()
               << Nepomuk::Vocabulary::NCO::photo();//this is also displayed as a pixmap

    foreach(QUrl propertyUrl, properties) { //create editor widgets
        Nepomuk::Types::Property property(propertyUrl);
        PropertyEditWidget *propertyWidget = new PropertyEditWidget(this,
                                                                    property,
                                                                    resource().property(propertyUrl));
        //connect(d->btnEditSave, SIGNAL(clicked()), propertyWidget, SLOT(toggleEditing()));

        QLabel *label = new QLabel();
        label->setText("<font color='#808080'>" + property.label() + "</font>:");//grey text
        label->setToolTip(property.uri().toString());

        formLayout->addRow(label, propertyWidget);
        addPropertyEditWidget(propertyWidget);
    }

    //vertical layout for picture
    QVBoxLayout *photoLayout = new QVBoxLayout();
    mainLayout->addLayout(photoLayout);

    d->photoLabel = new QLabel();
    photoLayout->addWidget(d->photoLabel, 1, Qt::AlignRight);
    loadContactPhoto();

    //set layout and focus policy
    setLayout(mainLayout);
}

void Nepomuk::ContactResourceEditPlugin::saveChanges()
{
    AbstractResourceEditPlugin::saveChanges();
    loadContactPhoto();//if changes are saved, reload photo
}

void Nepomuk::ContactResourceEditPlugin::loadContactPhoto()
{
    //TODO?: add frame and placeholder image (if none is defined)
    Nepomuk::Resource file = resource().property(Nepomuk::Vocabulary::NCO::photo()).toResource();
    QString fileUrl = file.property(Nepomuk::Vocabulary::NIE::url()).toString().remove("file://");
    //kDebug() << "*** setting photo:" << file.uri() << fileUrl;
    QPixmap pixmap;
    //kDebug() << "*** pixmap loded: " << pixmap.load(fileUrl);
    d->photoLabel->setPixmap(pixmap.scaled(QSize(64, 64), Qt::KeepAspectRatio));
}


void Nepomuk::ContactResourceEditPlugin::setupActions()
{
    AbstractResourceEditPlugin::setupActions();

    KAction *chat = new KAction(KIcon("im-user"), i18n("Start IM chat"), this);
    connect(chat, SIGNAL(triggered()), this, SLOT(startChat()));
    addAction(chat);
}


void Nepomuk::ContactResourceEditPlugin::startChat()
{
    //TODO: implement this

    kDebug() << "*** startChat: not implemented";
}
