/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "genericresourceeditplugin.h"

#include "../src/propertyeditwidget.h"
#include "../src/nepomukdataaccess.h"

#include <QLabel>
#include <QWidget>
#include <QVariant>
#include <QDate>
#include <QList>
#include <QSet>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QFormLayout>

#include <KPushButton>
#include <KDebug>

#include <Nepomuk/Query/QueryServiceClient>
#include <Nepomuk/Query/Result>
#include <Nepomuk/Resource>

#include <Soprano/Vocabulary/RDFS>
#include <Soprano/Vocabulary/NAO>

#include <KPluginFactory>
#include <KPluginLoader>

//K_PLUGIN_FACTORY(ResourceEditPluginFactory, registerPlugin<Nepomuk::GenericResourceEditPlugin>();)
//K_EXPORT_PLUGIN(ResourceEditPluginFactory("nepomuk_genericresourceeditplugin"))


class Nepomuk::GenericResourceEditPlugin::Private
{
public:
    QList<PropertyEditWidget*> propertyEditWidgets;
};

Nepomuk::GenericResourceEditPlugin::GenericResourceEditPlugin(QWidget *parent) :
        AbstractResourceEditPlugin(parent),
        d(new Private())
{
    //d->resource = resource;
    //kDebug() << "GenericResourceEditPlugin: " << this->resource().uri() << this->resource().label();
}

Nepomuk::GenericResourceEditPlugin::~GenericResourceEditPlugin()
{
    delete d;
}

void Nepomuk::GenericResourceEditPlugin::initGui()
{
    if (layout()) {
        layout()->deleteLater();
    }

    //add layout and focus policy
    QFormLayout *formLayout = new QFormLayout(this);
    formLayout->setSpacing(0);
    formLayout->setContentsMargins(11, 1, 1, 1);
    setFocusPolicy(Qt::StrongFocus);

    //-----------
    //add label and description
    Nepomuk::PropertyEditWidget *pewLabel = new PropertyEditWidget( this,
                                                                    Soprano::Vocabulary::NAO::prefLabel(),
                                                                    resource().genericLabel() );
    formLayout->addRow( "label", pewLabel );
    formLayout->labelForField(pewLabel)->setToolTip(Soprano::Vocabulary::NAO::prefLabel().toString());
    addPropertyEditWidget(pewLabel);

    Nepomuk::PropertyEditWidget *pewDescr = new PropertyEditWidget( this,
                                                                    Soprano::Vocabulary::NAO::description(),
                                                                    resource().genericDescription() );
    formLayout->addRow( "description", pewDescr );
    formLayout->labelForField(pewDescr)->setToolTip(Soprano::Vocabulary::NAO::description().toString());
    addPropertyEditWidget(pewDescr);

    //-----------
    //add all default properties
    QSet<QUrl> properties = resource().properties().keys().toSet();
    QList<QUrl> mandatoryProperties = NepomukDataAccess::mandatoryPropertiesForType(resource().resourceType());
    properties.unite(mandatoryProperties.toSet());
    properties.remove(Soprano::Vocabulary::NAO::prefLabel());//we've already added them above
    properties.remove(Soprano::Vocabulary::NAO::description());

    foreach(QUrl propertyUrl, properties) { //create editor widgets
        Nepomuk::Types::Property property(propertyUrl);
        Nepomuk::PropertyEditWidget *propertyWidget = new PropertyEditWidget(this,
                                                                    property,
                                                                    resource().property(propertyUrl));        
        formLayout->addRow(property.label(), propertyWidget);
        formLayout->labelForField(propertyWidget)->setToolTip(property.uri().toString());
        addPropertyEditWidget(propertyWidget);
    }    
}
