/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>
Copyright (C) 2010 Sebastian Trueg <trueg@kde.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef RESOURCEITEMDELEGATE_H
#define RESOURCEITEMDELEGATE_H

#include <KExtendableItemDelegate>

#include "nepomuksearchgui_export.h"

class ResourceEditWidget;

namespace Nepomuk
{

    /**
     * A Delegate for resource items. It provides a ResourceEditWidget to display and edit an active item.
     *
     * @author Oszkar Ambrus aoszkar@gmail.com
     */

class NEPOMUKSEARCHGUI_EXPORT ResourceItemDelegate : public KExtendableItemDelegate
{
    Q_OBJECT

public:

    enum ResourceDataRoles {
        ResourceUriRole = Qt::UserRole + 2,
        ResourceTypeRole = Qt::UserRole + 4,
        ResourceDescriptionRole = Qt::UserRole + 8};

    /**
     * Creates a new ResourceItemDelegate
     */
    ResourceItemDelegate(QAbstractItemView *parent = 0);
    ~ResourceItemDelegate();


    QSize sizeHint(const QStyleOptionViewItem & option, const QModelIndex & index) const;
    void paint(QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index) const;


    /**
     * @reimp
     *
     * Uses the ResourceEditWidgetFactory to create the corresponding editor for
     * the resource the given index
     */
    QWidget * createEditor(QWidget * parent, const QStyleOptionViewItem & option, const QModelIndex & index) const ;
    //void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;

    /**
      * @reimp
      */
    void updateEditorGeometry (QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const;


    /**
      * @reimp from KExtendableItemDelegate
      */
    virtual void updateExtenderGeometry(QWidget *extender, const QStyleOptionViewItem &option, const QModelIndex &index) const;

    /**
      * @reimp
      */
    bool editorEvent ( QEvent * event, QAbstractItemModel * model, const QStyleOptionViewItem & option, const QModelIndex & index );

public Q_SLOTS:
    void emitGeometryChanged();

private:
    class Private;
    Private * const d;
};

}


#endif // RESOURCEITEMDELEGATE_H
