/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef RESOURCEITEMVIEW_H
#define RESOURCEITEMVIEW_H

#include <QListView>

#include "nepomuksearchgui_export.h"

class QModelIndex;

namespace Nepomuk
{
// trueg: FIXME: would be nice if this class could handle data from any ResourceModel.
class NEPOMUKSEARCHGUI_EXPORT ResourceItemView : public QListView
{
    Q_OBJECT

    /**
     * A View for resource items. It provides a custom delegate to display
     * and edit properties of objects.
     *
     * @author Oszkar Ambrus aoszkar@gmail.com
     */

    public:
        /**
         * Creates a new ResourceItemView that doesn't automatically create an editor
         */
        ResourceItemView(QWidget *parent = 0);

    public Q_SLOTS:

        /**
         * Expands and contracts the item at index @p index.
         * For expanding the item it uses the editor widget provided by ResourceItemDelegate.
         */
        void itemActivated(const QModelIndex &index);

        /**
         * @reimp
         *
         * Contracts all items and calls QListView::reset()
         */
        void reset();

        /**
         * Refreshes the geometries of the items (and the editors).
         */
        void refreshGeometries();

    private Q_SLOTS:
        /**
         * Copies the label of the current resource to the clipboard
         */
        void copyLabelToClipboard();

        /**
         * Copies the URI of the current resource to the clipboard
         */
        void copyUriToClipboard();

        /**
         * Deletes the current resource from the RDF store
         */
        void deleteResource();
    };

}

#endif // RESOURCEITEMVIEW_H
