/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "xmlrpc.h"

#include <QtCore/QByteArray>
#include <QtCore/QDebug>
#include <QtCore/QEventLoop>
#include <QtCore/QFile>
#include <QtCore/QVariant>

#include <kxmlrpcclient/client.h>

XmlRpc::XmlRpc( const QUrl& service, QObject* parent )
    : QObject( parent )
{
    qDebug() << "XmlRpc::XmlRpc( " << service.toString().toUtf8().constData() << " )";
    loop = new QEventLoop( this );
    client = new KXmlRpc::Client( service, this );
    client->setUserAgent( "Test/1.0" );
}

void XmlRpc::findMapping( const QUrl& source, const QUrl& target, const QVariant& id )
{
    qDebug() << "XmlRpc::findMapping( " << source.toString().toUtf8().constData()
             << ", " << target.toString().toUtf8().constData() << " )";
    QList< QVariant > parameters;
    parameters.append( source.toString() );
    parameters.append( target.toString() );
    client->call( "RemoteMappingStorage.getMapping", parameters,
                  this, SLOT( gotData( const QList< QVariant >&, const QVariant& ) ),
                  this, SLOT( gotError( int, const QString&, const QVariant& ) ), id );
    loop->exec();
}

void XmlRpc::gotData( const QList< QVariant >& result, const QVariant& id )
{
    QByteArray data = result.first().toByteArray();
    const QString text = data.isEmpty() ? QString( "nothing" ) : QString( "something" );
    qDebug() << "XmlRpc::gotData( " << text.toUtf8().constData() << " )";

    if ( data.isEmpty() ) {
        loop->quit();
        return;
    }

    QFile zipFile( id.toString() + ".zip" );
    zipFile.open( QIODevice::WriteOnly );
    zipFile.write( data );
    zipFile.close();

    loop->quit();
}

void XmlRpc::gotError( int errorCode, const QString& errorString, const QVariant& id )
{
    qDebug() << "XmlRpc::gotError(" << errorCode << ", " << errorString.toUtf8().constData() << ")";
    loop->quit();
}

#include "xmlrpc.moc"
