/*
 * This file is part of Search Project.
 *
 * Copyright (C) 2007 Daniele Galdi <daniele.galdi@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <QDateTime>
#include <QAbstractItemModel>

#include <KGlobal>
#include <KLocale>
#include <KUrl>

#include "TreeView.h"

#include "ItemWidget.h"

ExpandedItemWidget::ExpandedItemWidget(const QModelIndex &index, QWidget *parent )
  :QWidget(parent), m_index(index)
{
  setupUi( this );

  const QAbstractItemModel* model = index.model();

  QModelIndex uriIndex = model->index(index.row(), 0, index );
  QVariant uri         = uriIndex.data( Qt::DisplayRole );
  m_where->setText( uri.toString() );

  KUrl url( uri.toString() );
  m_fileName->setText( url.fileName() );

  QModelIndex mimeTypeIndex = model->index(index.row(), 1, index );
  QVariant mimeType         = mimeTypeIndex.data( Qt::DisplayRole );
  m_mimeType->setText( mimeType.toString() );
  
  QModelIndex sizeIndex = model->index(index.row(), 2, index );
  QVariant size         = sizeIndex.data( Qt::DisplayRole );
  m_size->setText( size.toString() );

  QModelIndex mtimeIndex = model->index(index.row(), 3, index );
  QVariant mtime         = mtimeIndex.data( Qt::DisplayRole );

  QDateTime datetime =  QDateTime::fromTime_t( mtime.toInt() );
  m_modified->setText( KGlobal::locale()->formatDate( datetime.date() ) );
  
  connect( pushButton, SIGNAL( clicked() ),this, SLOT( switchClicked() ) );
}
  
ExpandedItemWidget::~ExpandedItemWidget()
{
  qDebug() << "~ExpandedItemWidget()";
}

void ExpandedItemWidget::switchClicked()
{
  emit stateChanged( m_index );
}

/* ClosedItemWidget */

ClosedItemWidget::ClosedItemWidget(const QModelIndex &index, QWidget *parent )
  :QWidget(parent), m_index(index)
{
  setupUi( this );

  const QAbstractItemModel* model = index.model();

  QModelIndex uriIndex = model->index(index.row(), 0, index );
  QVariant uri         = uriIndex.data( Qt::DisplayRole );
  KUrl url( uri.toString() );
  m_fileName->setText( url.fileName() );

  // For icons
  QModelIndex mimeTypeIndex = model->index(index.row(), 1, index );
  QVariant mimeType         = mimeTypeIndex.data( Qt::DisplayRole );
  
  QModelIndex mtimeIndex = model->index(index.row(), 3, index );
  QVariant mtime         = mtimeIndex.data( Qt::DisplayRole );

  QDateTime datetime =  QDateTime::fromTime_t( mtime.toInt() );
  m_date->setText( KGlobal::locale()->formatDate( datetime.date() ) );
  
  connect( pushButton, SIGNAL( clicked() ),this, SLOT( switchClicked() ) );
}
  
ClosedItemWidget::~ClosedItemWidget()
{
  qDebug() << "~ClosedItemWidget()";
}

void ClosedItemWidget::switchClicked()
{
  emit stateChanged( m_index, 1 );
}

#include "ItemWidget.moc"
