/*
 * This file is part of Search Project.
 *
 * Copyright (C) 2007 Daniele Galdi <daniele.galdi@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <QtGui>

#include <klocale.h>
#include <kmimetype.h>
#include <kiconloader.h>

#include "SearchMainWindow.h"

SearchMainWindow::SearchMainWindow( QWidget *parent )
  : QWidget(parent)
{
  setWindowTitle( i18n("Strigi") );

  m_model = new StrigiHitsModel();
  m_tree = new TreeView();
  m_tree->setModel( m_model );

  m_filter = new FilterBar();

  QHBoxLayout* hlayout = new QHBoxLayout( );
  hlayout->addWidget( m_tree );
  hlayout->addWidget( m_filter );

  m_searchBar = new SearchBar();
  connect( m_searchBar->search,
	   SIGNAL( returnPressed() ),
	   this,
	   SLOT( returnPressed() ) );

  QVBoxLayout* vlayout = new QVBoxLayout( this );
  vlayout->addWidget( m_searchBar );
  vlayout->addLayout( hlayout );

  //---------------EXPANDALL

  connect( m_model,
	   SIGNAL( modelReset() ),
	   m_tree,
	   SLOT( expandAll() ) );

  //------------------------------------

  connect( &m_client,
	   SIGNAL( gotHits( const QString&, int, const QList<StrigiHit>& ) ),
	   this,
	   SLOT( handleHits( const QString&, int, const QList<StrigiHit>& ) ) );

  connect( &m_client,
	   SIGNAL( countedQuery( const QString&, int ) ),
	   this,
	   SLOT( handleCount( const QString &, int ) ) );

  //-----------------------------------

  connect( m_filter->groupFlat,
	   SIGNAL( leftClickedUrl( const QString& ) ),
	   m_model,
	   SLOT( groupBy( const QString & ) ) );		  

  connect( m_filter->groupDate,
	   SIGNAL( leftClickedUrl( const QString& ) ),
	   m_model,
	   SLOT( groupBy( const QString & ) ) );		  

  connect( m_filter->groupKind,
	   SIGNAL( leftClickedUrl( const QString& ) ),
	   m_model,
	   SLOT( groupBy( const QString & ) ) );		  

  connect( m_filter->sortName,
	   SIGNAL( leftClickedUrl( const QString& ) ),
	   this,
	   SLOT( sortBy( const QString & ) ) );		  

  connect( m_filter->sortKind,
	   SIGNAL( leftClickedUrl( const QString& ) ),
	   this,
	   SLOT( sortBy( const QString & ) ) );		  

  connect( m_filter->sortDate,
	   SIGNAL( leftClickedUrl( const QString& ) ),
	   this,
	   SLOT( sortBy( const QString & ) ) );		  
  
  //-----------------------------------
  
  setMinimumWidth( 800 );
  setMinimumHeight( 600 );
}
  
SearchMainWindow::~SearchMainWindow()
{
}	

void SearchMainWindow::returnPressed()
{
  m_searchBar->count->hide();
  QString query = m_searchBar->search->text();

  setWindowTitle( i18n("Strigi").append(" - %1 ").arg( query ) );

  m_client.clearRequests(StrigiAsyncClient::Query);

  m_client.addCountQuery( query );
  m_client.addGetQuery(query, 10, 0);

  m_queryHits = query;
  m_queryCount = query;
}

void SearchMainWindow::handleHits( const QString &query, int offset, const QList<StrigiHit> &hits )
{
  if ( m_queryHits != query )
  {
    return; 
  }

  m_model->setHits( hits );

  kDebug() << "SearchMainWindow::handleHits " << query << endl;
  kDebug() << "SearchMainWindow::handleHits.size " << hits.size() << endl;
}

void SearchMainWindow::handleCount( const QString &query, int count )
{
  if ( m_queryCount != query )
  {
    return; 
  }

  kDebug() << "SearchMainWindow::handleCount " << query << count << endl;
  
  m_searchBar->count->setText( QString("%1 Results").arg( count ) );
  m_searchBar->count->show();
}

void SearchMainWindow::sortBy( const QString& sortBy )
{
  kDebug() << "SearchMainWindow::sortBy " << sortBy << endl;
}

#include "SearchMainWindow.moc"
