/***************************************************************************
 *   Copyright 2008 by Thomas Schildknecht <danakil.kde@gmail.com>         *
 *   Copyright 2010 by Stefan Majewsky <majewsky@gmx.net>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#ifndef AWC_BUTTON_H
#define AWC_BUTTON_H

#include <Plasma/IconWidget>

namespace Awc
{
    enum ButtonType
    {
        SeparatorButton = 0,
        CloseButton,
        MaximizeButton,
        MinimizeButton,
        ShadeButton,
        KeepBelowButton,
        KeepAboveButton,
        ButtonTypeCount
    };

    struct ButtonMetadata
    {
        public:
            ButtonMetadata(Awc::ButtonType buttonType);

            QIcon renderedIcon() const;

            Awc::ButtonType m_buttonType;
            QString m_elementIdentifier;
            QString m_actionString, m_actionText;
    };

    class WindowManager;

    class Button : public Plasma::IconWidget
    {
        Q_OBJECT
        public:
            Button(Awc::ButtonType buttonType, Awc::WindowManager* windowManager, QGraphicsItem* parent = 0);

            Awc::ButtonType buttonType() const;
            bool isPushed() const;
        public Q_SLOTS:
            void setEnabled(bool enabled);
            void setPushed(bool pushed);
        Q_SIGNALS:
            void actionTriggered(const QString& actionIdentifier);
        private Q_SLOTS:
            void slotClicked();
        private:
            void updateAppearance();

            Awc::ButtonMetadata m_metadata;
            bool m_pushed;
    };
}

#endif // AWC_BUTTON_H
