/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef PLASMA_CONTACTS_H
#define PLASMA_CONTACTS_H

#include <QtCore/QString>
#include <QtCore/QList>
#include <QtCore/QHash>
#include <QtGui/QListWidgetItem>

#include <Plasma/Applet>
#include <Plasma/DataEngine>

#include "ui_contactsConfig.h"
#include "ui_displayConfig.h"

// Forward declarations
namespace Plasma
{
class Svg;
class FlashingLabel;
}
class QGraphicsLinearLayout;
class ContactWidget;
class GlobalWidget;

/**
 * Small helper class to set list item text from id (quick to get)
 * to displayName of contacts (might take longer)
 *
 * @todo: I'd rather have this class located elsewhere
 */
class LabelUpdater: public QObject
{
Q_OBJECT

public:
	/** Constructor */
	LabelUpdater(QListWidgetItem *item, const QString &source, Plasma::DataEngine *engine) :
		m_item(item), m_source(source), m_engine(engine)
	{
		// nothing to do
	}

	/** Destructor */
	~LabelUpdater()
	{
	}

	private Q_SLOTS:

		/** Called by the data engine */
		void dataUpdated(const QString &sourceName, const Plasma::DataEngine::Data &data)
		{
			Q_UNUSED(sourceName);

			QString displayName = data["name"].toString();
			if (!displayName.isEmpty())
			{
				QString email = data["email"].toString();
				QString im = data["instant_messaging_id"].toString();
				QString text = displayName;
				if (!email.isEmpty())
				{
					text = displayName + " (" + email + ")";
				}
				else if (!im.isEmpty())
				{
					text = displayName + " (" + im + ")";
				}

				m_item->setText(text);

				// We don't need more updates
				m_engine->disconnectSource(m_source, this);
			}
		}

	private:
		/** Widget to be updated */
		QListWidgetItem *m_item;

		/** Source name */
		const QString m_source;

		/** Source that updates us */
		Plasma::DataEngine *m_engine;
};

class PlasmaContacts: public Plasma::Applet
{
Q_OBJECT

public:
	/** Constructor */
    PlasmaContacts(QObject *parent, const QVariantList &args)									;

	/** Destructor */
~	PlasmaContacts();

	/** Reimplemented from Plasma::Applet */
	void init();

	/** Reimplemented from Plasma::Applet */
	void paintInterface(QPainter *p,
			const QStyleOptionGraphicsItem *option, const QRect &contentsRect);

public Q_SLOTS:
	/** Reimplemented from Plasma::Applet */
	void dataUpdated(const QString &name, const Plasma::DataEngine::Data &data);

	/** Reimplemented from Plasma::Applet */
	void createConfigurationInterface(KConfigDialog *parent);

protected Q_SLOTS:
	/** Reimplemented from Plasma::Applet */
	void configAccepted();

private:
	/** Load and apply configuration data */
	void load();

	/** Background theme (unused) */
	Plasma::Svg *m_theme;

	/** Data engine we're connected to */
	Plasma::DataEngine *m_engine;

	/** Global layout */
	QGraphicsLinearLayout *m_layout;

	/** All contacts (rows) currently displayed */
	QHash<QString,ContactWidget*> m_contactWidgets;

	/** Contact configuration widget */
	Ui::ContactsConfig contactsPage;

	/** Display configuration widget */
	Ui::DisplayConfig displayPage;

	/** Configuration hint */
	Plasma::FlashingLabel *m_flash;

	/** Config widget labels being updated */
	QList<LabelUpdater*> m_labelUpdaters;

	/** Global widgets currently shown */
	QList<GlobalWidget*> m_globalWidgets;
};

#endif // PLASMA_CONTACTS_H
