/*
 *   Copyright (C) 2008 Dennis Jansen <Dennis.Jansen@web.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 or later
 *   as published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

// Here we avoid loading the header multiple times
#ifndef plasma_darkstat_HEADER
#define plasma_darkstat_HEADER

// We need the Plasma Applet headers
#include <Plasma/Applet>
#include <Plasma/DataEngine>

#include <kio/job.h>

#include <QTimer>

#include <QDialog>
#include <QGraphicsLinearLayout>
#include "ui_widget.h"

namespace Plasma {
    class Label;
}

// Define our plasma Applet
class PlasmaDarkstat : public Plasma::Applet
{
   Q_OBJECT
   public:
      // Basic Create/Destroy
      PlasmaDarkstat(QObject *parent, const QVariantList &args);
      ~PlasmaDarkstat();

      void init();
      void constraintsUpdated(Plasma::Constraints);

/* Q_SLOTS to be able to connect http signals */
   private Q_SLOTS:
      void Download();
      void DownloadFinished();

   protected slots:
        void configAccepted();

   protected:
        void createConfigurationInterface(KConfigDialog *parent);

   private:
/* mine */
      // functions
      void Repeat();
      void Download_init();
      void UpdateText();
      void err(QString);

      // http
      KIO::StoredTransferJob *job;
      KUrl url;

      // strings
      QString graphs_xml; /* the graphs file content */
      QString text; /* the text extracted from graphs_xml */

      // loop
      QTimer *UpdateLoopTimer; /* timer which sleeps for UpdateLoopInterval ms
                              and then calls Download again() */
      bool KeepGoing;
      int UpdateLoopInterval;

      QTime time;

      // display labels
      Plasma::Label
      *m_TextPackets, *m_TextKB,
      *m_TextKBpsi, *m_TextKBpso;

      QGraphicsLinearLayout *m_layoutv_main, *m_layouth_misc, *m_layouth_IO;

      Ui::darkstatConfig ui;
};

K_EXPORT_PLASMA_APPLET(Darkstat, PlasmaDarkstat)
#endif


void verboseLog(QString);
