/***************************************************************************
 *   Copyright (C) 2010 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "context_menu_p.h"

#include "service_action.h"
#include "tasks_roles.h"

#include <Plasma/DataEngine>
#include <KIcon>

ContextMenu::ContextMenu(const QString &menuName, const QIcon &menuIcon,
                         const QList<QVariant> &menuDescription,
                         const QPersistentModelIndex &modelIndex,
                         Plasma::DataEngine *tasksEngine,
                         Plasma::DataEngine *statusNotifierItemEngine,
                         QWidget *parent)
    : QMenu(parent), m_tasksEngine(tasksEngine), m_statusNotifierItemEngine(statusNotifierItemEngine), m_modelIndex(modelIndex)
{
    setTitle(menuName);
    if (!menuIcon.isNull()) {
        setIcon(menuIcon);
    }

    for (int i = 0; i < menuDescription.size(); ++i) {
        const QVariantMap &menuItemDescription(menuDescription.at(i).toMap());
        if (menuItemDescription.contains("isSeparator") && menuItemDescription["isSeparator"].toBool()) {
            addSeparator();
        } else {
            if (menuItemDescription.contains("subMenuItems")) {
                addMenu(new ContextMenu(menuItemDescription["menuName"].toString(), QIcon(),
                                        menuItemDescription["subMenuItems"].toList(),
                                        m_modelIndex, m_tasksEngine, m_statusNotifierItemEngine,
                                        parent));
            } else {
                addServiceAction(menuItemDescription);
            }
        }
    }
}

void ContextMenu::addServiceAction(const QVariantMap &actionDescription)
{
    Plasma::Service *taskService = NULL;
    const QVariant &taskSourceName(m_modelIndex.data(TasksModel::TaskSourceName));
    if (taskSourceName.canConvert<QString>()) {
        taskService = m_tasksEngine->serviceForSource(m_modelIndex.data(TasksModel::TaskSourceName).toString());
    }

    Plasma::Service *statusItemService = NULL;
    const QVariant &statusItemSourceNames(m_modelIndex.data(TasksModel::StatusItemSourceNames));
    if (statusItemSourceNames.canConvert<QList<QVariant> >() && !statusItemSourceNames.toList().empty()) {
        statusItemService = m_statusNotifierItemEngine->serviceForSource(statusItemSourceNames.toList().first().toString());
    }

    addAction(new ServiceAction(this, taskService, statusItemService, actionDescription));
}
