/*****************************************************************************
 *   Copyright (C) 2009-2010 by Matthieu Gallien <matthieu_gallien@yahoo.fr> *
 *   Copyright (C) 2007 by Robert Knight <robertknight@gmail.com>            *
 *   Copyright (C) 2008 by Alexis Ménard <darktears31@gmail.com>             *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License, or       *
 *   (at your option) any later version.                                     *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .          *
 *****************************************************************************/

#include "dataengines_tasks.h"

#include "tasks_model.h"
#include "tasks_view.h"
#include "tasks_roles.h"

#include <Plasma/FrameSvg>
#include <KIconLoader>
#include <QGraphicsLinearLayout>
#include <QSortFilterProxyModel>

DataEnginesTasks::DataEnginesTasks(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
      m_model(NULL),
      m_graphicsView(NULL),
      m_taskItemBackground(0)
{
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
}


DataEnginesTasks::~DataEnginesTasks()
{
    delete m_model;
}

void DataEnginesTasks::init()
{
    QSortFilterProxyModel *newModel = new QSortFilterProxyModel(this);
    newModel->setSourceModel(new FlatTasksModel(dataEngine("tasks"), dataEngine("statusnotifieritem"), this));
    newModel->setFilterRole(TasksModel::IsOnCurrentDesktopRole);
    newModel->setDynamicSortFilter(true);
    newModel->setFilterRegExp("true");
    m_model = newModel;

    QGraphicsLinearLayout *tasksLayout = new QGraphicsLinearLayout(this);

    m_graphicsView = new TasksView(this, dataEngine("tasks"), dataEngine("statusnotifieritem"), this);
    m_graphicsView->setModel(m_model);
    tasksLayout->addItem(m_graphicsView);
    tasksLayout->setSpacing(0.0);
    tasksLayout->setContentsMargins(0, 0, 0, 0);

    setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
    setContentsMargins(0, 0, 0, 0);
}

/**
 * \todo Implement me
 */
void DataEnginesTasks::constraintsEvent(Plasma::Constraints constraints)
{
    if (constraints & Plasma::FormFactorConstraint && m_graphicsView) {
        if (formFactor() == Plasma::Horizontal) {
            m_graphicsView->setOrientation(Qt::Horizontal);
        } else {
            m_graphicsView->setOrientation(Qt::Vertical);
        }
    }
}

Plasma::FrameSvg* DataEnginesTasks::itemBackground()
{
    if (!m_taskItemBackground) {
        m_taskItemBackground = new Plasma::FrameSvg(this);
        m_taskItemBackground->setImagePath("widgets/tasks");
        m_taskItemBackground->setCacheAllRenderedFrames(true);
    }

    return m_taskItemBackground;
}

void DataEnginesTasks::resizeItemBackground(const QSizeF &size)
{
  //kDebug();
    if (!m_taskItemBackground) {
        itemBackground();
    }

    if (m_taskItemBackground->frameSize() == size) {
        //kDebug() << "Error2";
        return;
    }

    m_taskItemBackground->resizeFrame(size);

    QString oldPrefix = m_taskItemBackground->prefix();
    m_taskItemBackground->setElementPrefix("normal");
    //get the margins now
    m_taskItemBackground->getMargins(m_leftMargin, m_topMargin, m_rightMargin, m_bottomMargin);

    // the offscreen margins are always whatever the svg naturally is
    m_offscreenLeftMargin = m_leftMargin;
    m_offscreenTopMargin = m_topMargin;
    m_offscreenRightMargin = m_rightMargin;
    m_offscreenBottomMargin = m_bottomMargin;

    //if the task height is too little shrink the top and bottom margins
    if (size.height() - m_topMargin - m_bottomMargin < KIconLoader::SizeSmall) {
        m_topMargin = m_bottomMargin = qMax(1, int((size.height() - KIconLoader::SizeSmall)/2));
    }
    m_taskItemBackground->setElementPrefix(oldPrefix);
}

void DataEnginesTasks::needsVisualFocus()
{
    emit activate();
}

K_EXPORT_PLASMA_APPLET(dataengines_tasks, DataEnginesTasks)
