/***************************************************************************
 *   Copyright (C) 2010 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "service_action.h"

#include <KConfigGroup>
#include <Plasma/Service>

ServiceAction::ServiceAction(QObject *parent,
                             Plasma::Service *taskService,
                             Plasma::Service *statusItemService,
                             const QVariantMap &parameters)
    : QAction(parent),
      m_service(parameters["useTaskService"].toBool() ? taskService : statusItemService),
      m_operationName(parameters["operationName"].toString()),
      m_extraOperationParameters(parameters["extraOperationParameters"].toMap())
{
    setText(parameters["menuName"].toString());

    if (parameters.contains("icon")) {
        setIcon(parameters["icon"].value<QIcon>());
    }

    if (parameters.contains("checked")) {
        setCheckable(true);
        setChecked(parameters["checked"].toBool());
    }

    if (parameters.contains("enabled")) {
        setEnabled(parameters["enabled"].toBool());
    }

    connect(this, SIGNAL(triggered()), this, SLOT(activeService()));
}

void ServiceAction::activeService()
{
    Q_ASSERT(m_service);

    KConfigGroup params = m_service->operationDescription(m_operationName);
    for(QMap<QString, QVariant>::const_iterator it = m_extraOperationParameters.constBegin(); it != m_extraOperationParameters.constEnd(); ++it) {
        params.writeEntry(it.key(), it.value());
    }
    m_service->startOperationCall(params);
}
