/***************************************************************************
 *   Copyright (C) 2009 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "tasks_grouping_model.h"
#include "tasks_roles.h"

#include <KIcon>

// Q_DECLARE_METATYPE(QModelIndex)

TasksGroupingModel::TasksGroupingModel(Plasma::DataEngine *tasksEngine,
                                       const QString &propertyName,
                                       QObject *parent) :
    QAbstractItemModel(parent),
    m_tasksEngine(tasksEngine),
    m_hierarchy(),
    m_tasksData(),
    m_propertyName(propertyName)
{
    kDebug();
    connect(m_tasksEngine, SIGNAL(sourceAdded(const QString&)),
            this, SLOT(taskAdded(const QString&)));
    connect(m_tasksEngine, SIGNAL(sourceRemoved(const QString&)),
            this, SLOT(taskRemoved(const QString&)));
    m_tasksEngine->connectAllSources(this);
}

TasksGroupingModel::~TasksGroupingModel()
{
    kDebug();
}

QVariant TasksGroupingModel::data(const QModelIndex &index, int role) const
{
    kDebug();
    kDebug() << "index:" << index << "role:" << role;
    if (index.isValid()) {
        if (index.internalId() == 0) {
            int categoryIndex = index.row();
            return categoryData(role, categoryIndex);
        } else {
            kDebug() << m_hierarchy;
            int categoryIndex = index.internalId() - 1;
            Q_ASSERT(categoryIndex >= 0 && categoryIndex < m_hierarchy.size());
            Q_ASSERT(index.row() >= 0 && index.row() < m_hierarchy.at(categoryIndex).size());
            return taskData(m_hierarchy.at(categoryIndex).at(index.row()), role);
        }
    } else {
        return QVariant();
    }
}

QVariant TasksGroupingModel::categoryData(int role, int categoryIndex) const
{
    kDebug();
    switch (role)
    {
        case Qt::DisplayRole:
            return m_tasksData[m_hierarchy.at(categoryIndex).at(0)][m_propertyName];
        case Qt::DecorationRole:
            return m_tasksData[m_hierarchy.at(categoryIndex).at(0)]["icon"];
        case TasksModel::IsOnCurrentDesktopRole:
            return true;
            break;
        default:
            return QVariant();
    }
}

QVariant TasksGroupingModel::taskData(const QString &taskName, int role) const
{
    kDebug();
    kDebug() << "taskName:" << taskName << "role:" << role;

    switch (role)
    {
        case Qt::DisplayRole:
            if (m_tasksData[taskName]["startup"].toBool()) {
                return m_tasksData[taskName]["text"];
            } else {
                return m_tasksData[taskName]["visibleName"];
            }
            break;
        case Qt::DecorationRole:
            return m_tasksData[taskName]["icon"];
            break;
        case TasksModel::IsMinimizedRole:
            return m_tasksData[taskName]["minimized"];
            break;
        case TasksModel::IsMaximizedRole:
            return m_tasksData[taskName]["maximized"];
            break;
        case TasksModel::IsShadedRole:
            return m_tasksData[taskName]["shaded"];
            break;
        case TasksModel::DesktopNumberRole:
            return m_tasksData[taskName]["desktop"];
            break;
        case TasksModel::IsOnCurrentDesktopRole:
            return m_tasksData[taskName]["onCurrentDesktop"];
            break;
        case TasksModel::IsActive:
            return m_tasksData[taskName]["active"];
            break;
        case TasksModel::IsDemandingAttention:
            return m_tasksData[taskName]["demandsAttention"];
            break;
        case TasksModel::BackgroundPrefix:
            return m_tasksData[taskName]["backgroundPrefix"];
            break;
        case TasksModel::ClassName:
            return m_tasksData[taskName]["className"];
            break;
        case TasksModel::ClassClass:
            if (m_tasksData[taskName]["startup"].toBool()) {
                return m_tasksData[taskName]["text"];
            } else {
                return m_tasksData[taskName]["classClass"];
            }
            break;
        case TasksModel::IsStartup:
            return m_tasksData[taskName]["startup"];
            break;
        case TasksModel::TaskSourceName:
            return taskName;
            break;
    }

    return QVariant();
}

bool TasksGroupingModel::hasChildren(const QModelIndex &index) const
{
    kDebug();
    if (index.isValid()) {
        if (index.parent().isValid()) {
            return false;
        } else {
            return true;
        }
    } else {
        return true;
    }
}

int TasksGroupingModel::rowCount(const QModelIndex &index) const
{
    kDebug();
    kDebug() << "enter" << index;
    if (index.isValid()) {
        if (index.internalId() != 0) {
            kDebug() << "leave with valid parent";
            return 0;
        } else {
            return m_hierarchy[index.row()].size();
        }
    } else {
        kDebug() << "leave with invalid index:" << m_hierarchy.size();
        return m_hierarchy.size();
    }
}

int TasksGroupingModel::columnCount(const QModelIndex &index) const
{
    kDebug();
    Q_UNUSED(index);
    return 1;
}

QModelIndex TasksGroupingModel::index(int row, int column, const QModelIndex &parent) const
{
    kDebug();
//     kDebug() << "enter" << row << column << parent;
    if (parent.isValid()) {
        if (parent.parent().isValid()) {
//             kDebug() << "leave";
            return QModelIndex();
        } else {
            if (column != 0) {
//                 kDebug() << "leave";
                return QModelIndex();
            }
            if (row < 0 || row >= m_hierarchy[parent.row()].size()) {
//                 kDebug() << "leave";
                return QModelIndex();
            }
//             kDebug() << "leave";
            return createIndex(row, column, parent.row() + 1);
        }
    } else {
        if (column != 0) {
//             kDebug() << "leave";
            return QModelIndex();
        }
        if (row < 0 || row >= m_hierarchy.size()) {
//             kDebug() << "leave";
            return QModelIndex();
        }
//         kDebug() << "leave";
        return createIndex(row, column, 0);
    }
}

QModelIndex TasksGroupingModel::parent(const QModelIndex &childItem) const
{
    kDebug();
//     kDebug() << "enter" << childItem;
    if (childItem.isValid()) {
        if (childItem.internalId() == 0) {
//             kDebug() << "leave";
            return QModelIndex();
        } else {
//             kDebug() << "leave";
            return createIndex(int(childItem.internalId() - 1), 0);
        }
    } else {
//         kDebug() << "leave";
        return QModelIndex();
    }
}

void TasksGroupingModel::setGroupingProperty(const QString &propertyName)
{
    kDebug();
    kDebug() << "enter";
    m_propertyName = propertyName;
    resetEverything();
    kDebug() << "leave";
}

void TasksGroupingModel::resetEverything()
{
    kDebug();
    kDebug() << "enter";
    beginResetModel();
    m_hierarchy.clear();
    endResetModel();
    for (QHash<QString, Plasma::DataEngine::Data>::iterator itTaskData = m_tasksData.begin();
         itTaskData != m_tasksData.end(); ++itTaskData) {
        newSourceItem(itTaskData.key(), itTaskData.value());
    }
    kDebug() << "leave";
}

void TasksGroupingModel::newSourceItem(const QString &taskName,
                                       const Plasma::DataEngine::Data &taskData)
{
    kDebug();
    kDebug() << "enter" << taskName << taskData;

    const QVariant &newItemCategory(taskData[m_propertyName]);
    if (!newItemCategory.isValid()) {
        kDebug() << "leave with empty category";
        return;
    }

    bool inserted = false;
    {
        hierarchy_type::iterator itCategory;
        int categoryIndex;
        for (itCategory = m_hierarchy.begin(), categoryIndex = 0;
             itCategory != m_hierarchy.end(); ++itCategory, ++categoryIndex) {
            const QVariant &currentCategory(m_tasksData[itCategory->at(0)][m_propertyName]);

            if (newItemCategory == currentCategory) {
                kDebug() << "found already existing category:" << currentCategory;
                QModelIndex parent = index(categoryIndex, 0);
                kDebug() << parent << "has a new row:" << itCategory->size();
                beginInsertRows(parent, itCategory->size(), itCategory->size());
                itCategory->append(taskName);
                m_tasksData[taskName] = taskData;
                endInsertRows();
                kDebug() << "finished inserting the item in existing category";
                inserted = true;
            }
        }
    }

    if (!inserted) {
        kDebug() << "create new category";
        createNewCategory(newItemCategory, taskName, taskData);
    }
    kDebug() << "leave";
}

void TasksGroupingModel::createNewCategory(const QVariant &theCategory,
                                           const QString &taskName,
                                           const Plasma::DataEngine::Data &taskData)
{
    kDebug();
    kDebug() << "enter" << theCategory;
    const int newCategoryIndex = m_hierarchy.size();
    beginInsertRows(QModelIndex(), newCategoryIndex, newCategoryIndex);
    m_hierarchy.push_back(QList<QString>());
    endInsertRows();
    kDebug() << index(newCategoryIndex, 0);
    beginInsertRows(index(newCategoryIndex, 0), 0, 0);
    m_hierarchy.last().append(taskName);
    m_tasksData[taskName] = taskData;
    endInsertRows();
    kDebug() << "leave";
}

void TasksGroupingModel::taskAdded(const QString &taskName)
{
    kDebug();
    kDebug() << taskName;
    m_tasksEngine->connectSource(taskName, this);
}

void TasksGroupingModel::dataUpdated(const QString &taskName, const Plasma::DataEngine::Data &taskData)
{
    kDebug();
    if (m_tasksData.contains(taskName)) {
        m_tasksData[taskName] = taskData;
        const QModelIndex &modifiedIndex(indexFromTaskName(taskName));
        emit dataChanged(modifiedIndex, modifiedIndex);
        updateBackgroundPrefix(taskName, taskData);
    } else {
        newSourceItem(taskName, taskData);
    }
}

void TasksGroupingModel::updateBackgroundPrefix(const QString &taskName, const Plasma::DataEngine::Data &taskData)
{
    kDebug();
    if (taskData.contains("demandsAttention")) {
        if (taskData["demandsAttention"].toBool()) {
            if (m_tasksData[taskName]["backgroundPrefix"] != "attention") {
                m_tasksData[taskName]["backgroundPrefix"] = "attention";
                const QModelIndex &modifiedIndex(indexFromTaskName(taskName));
                emit dataChanged(modifiedIndex, modifiedIndex);
            }
        } else {
            if (taskData["active"].toBool()) {
                if (m_tasksData[taskName]["backgroundPrefix"] != "focus") {
                    m_tasksData[taskName]["backgroundPrefix"] = "focus";
                    const QModelIndex &modifiedIndex(indexFromTaskName(taskName));
                    emit dataChanged(modifiedIndex, modifiedIndex);
                }
            } else {
                if (m_tasksData[taskName]["backgroundPrefix"] != "normal") {
                    m_tasksData[taskName]["backgroundPrefix"] = "normal";
                    const QModelIndex &modifiedIndex(indexFromTaskName(taskName));
                    emit dataChanged(modifiedIndex, modifiedIndex);
                }
            }
        }
    } else {
        if (m_tasksData[taskName]["backgroundPrefix"] != "normal") {
            m_tasksData[taskName]["backgroundPrefix"] = "normal";
            const QModelIndex &modifiedIndex(indexFromTaskName(taskName));
            emit dataChanged(modifiedIndex, modifiedIndex);
        }
    }
}

QModelIndex TasksGroupingModel::indexFromTaskName(const QString &taskName) const
{
    kDebug();
    const QVariant &taskCategory(m_tasksData[taskName][m_propertyName]);

    {
        hierarchy_type::const_iterator itCategory;
        int categoryIndex;
        for (itCategory = m_hierarchy.begin(), categoryIndex = 0;
             itCategory != m_hierarchy.end(); ++itCategory, ++categoryIndex) {
            const QVariant &currentCategory(m_tasksData[itCategory->at(0)][m_propertyName]);

            if (taskCategory == currentCategory) {
                for (int i = 0; i < itCategory->size(); ++i) {
                    if (itCategory->operator[](i) == taskName) {
                        return index(i, 0, index(categoryIndex, 0));
                    }
                }
            }
        }
    }

    return QModelIndex();
}

void TasksGroupingModel::taskRemoved(const QString &taskName)
{
    kDebug();
    kDebug() << "taskName:" << taskName;
    const QModelIndex &removedIndex = indexFromTaskName(taskName);
    kDebug() << "removedIndex:" << removedIndex;
    if (!removedIndex.isValid() || !removedIndex.parent().isValid()) {
        kDebug() << "leaving with a preliminary return for no reasons";
        return;
    }

    int categoryIndex = removedIndex.parent().row();
    kDebug() << "beginRemoveRows(" << removedIndex.parent() << "," << removedIndex.row() << "," << removedIndex.row() << ")";
    kDebug() << persistentIndexList();
    beginRemoveRows(removedIndex.parent(), removedIndex.row(), removedIndex.row());
    m_hierarchy[categoryIndex].removeAt(removedIndex.row());
    m_tasksData.remove(taskName);
    endRemoveRows();
    kDebug() << "endRemoveRows()";
    kDebug() << persistentIndexList();
    if (m_hierarchy[categoryIndex].size() == 0) {
        kDebug() << "beginRemoveRows(" << removedIndex.parent().parent() << "," << categoryIndex << "," << categoryIndex << ")";
        beginRemoveRows(removedIndex.parent().parent(), categoryIndex, categoryIndex);
        m_hierarchy.removeAt(categoryIndex);
        QModelIndexList changedIndexes;
        QModelIndexList newIndexes;
        const QModelIndexList &storedIndexes(persistentIndexList());
        for (int i = 0; i < storedIndexes.size(); ++i) {
            const QModelIndex &currentIndex(storedIndexes[i]);
            if (currentIndex.parent().isValid()) {
                if (currentIndex.internalId() > categoryIndex + 1) {
                    changedIndexes << currentIndex;
                    newIndexes << index(currentIndex.row(), 0, index(currentIndex.internalId() - 2, 0));
                }
            }
        }
        if (!changedIndexes.empty()) {
            changePersistentIndexList(changedIndexes, newIndexes);
        }
        endRemoveRows();
        kDebug() << "endRemoveRows()";
        kDebug() << persistentIndexList();
    }
}
