/***************************************************************************
 *   Copyright (C) 2008 by Sebastian Kuegler <sebas@kde.org>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "cell.h"
#include <limits.h>

#include <QPainter>
#include <QBrush>
#include <QGraphicsSceneHoverEvent>
#include <plasma/theme.h>

Cell::Cell(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args)
{
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    setSizePolicy(QSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored));
    setBackgroundHints(NoBackground);
    setMinimumSize(8, 8);
    setMaximumSize(INT_MAX, INT_MAX);
    resize(64, 64);
    m_isHovered = false;
}

Cell::~Cell ()
{
}

void Cell::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
    //kDebug() << "Hovered";
    m_isHovered = true;
    Applet::hoverEnterEvent(event);
    update();
}

void Cell::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
    //kDebug() << "UnHovered";
    m_isHovered = false;
    Applet::hoverLeaveEvent(event);
    update();
}

void Cell::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
    Q_UNUSED( option );

    p->setRenderHint(QPainter::SmoothPixmapTransform);
    p->setRenderHint(QPainter::Antialiasing);

    QColor defaultTextColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
    defaultTextColor.setAlphaF(0.7);

    QColor fillColor = defaultTextColor;
    fillColor.setAlphaF(0.5);
    if (!m_isHovered) {
        fillColor.setAlphaF(0.3);
    }
    p->setPen(defaultTextColor);
    p->setBrush(fillColor);
    p->drawRoundedRect(contentsRect, .5, .5, Qt::RelativeSize);

}

#include "cell.moc"
