/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#include "avrecordbin.h"
#include <KDebug>
#include "libqtgstreamer/qgstpad.h"
#include "libqtgstreamer/qgstghostpad.h"
#include "libqtgstreamer/qgstelementfactory.h"
#include <gst/gst.h>
using namespace QtGstreamer;

class AVRecordBinPrivate
{
    public:
        QGstElementPtr fileSink;
        QGstElementPtr videoRecordQueue;
};

AVRecordBinPtr AVRecordBin::createAVRecordBin()
{
    QGstElementPtr videoQueue = QGstElementFactory::make("queue");
    QGstElementPtr colorSpace = QGstElementFactory::make("ffmpegcolorspace");
    QGstElementPtr videoRate = QGstElementFactory::make("videorate");
    QGstElementPtr videoEncoder = QGstElementFactory::make("theoraenc");
    QGstElementPtr videoQueue2 = QGstElementFactory::make("queue");
    QGstElementPtr avMuxer = QGstElementFactory::make("oggmux");
    QGstElementPtr fileSink = QGstElementFactory::make("filesink");
    QGstElementPtr audioSource = QGstElementFactory::make("autoaudiosrc");
    QGstElementPtr audioQueue = QGstElementFactory::make("queue");
    QGstElementPtr audioQueue2 = QGstElementFactory::make("queue");
    QGstElementPtr audioQueue3 = QGstElementFactory::make("queue");
    QGstElementPtr audioConvert = QGstElementFactory::make("audioconvert");
    QGstElementPtr audioEncoder = QGstElementFactory::make("vorbisenc");

    if ( !videoQueue || !colorSpace || !videoEncoder || !avMuxer || !fileSink) {
        return AVRecordBinPtr();
    }
    videoEncoder->setProperty("keyframe-auto", FALSE);
    videoEncoder->setProperty("keyframe-freq", 20);
    videoEncoder->setProperty("keyframe-force", 20); 
    videoEncoder->setProperty("keyframe-mindistance", 3);
    //videoEncoder->setProperty("quality",60);

    AVRecordBin *videoRecordBin = new AVRecordBin();
    videoRecordBin->d->fileSink = fileSink;
    videoRecordBin->d->fileSink->setProperty("location", QByteArray("dodol.ogg"));
    videoRecordBin->d->fileSink->setProperty("async",FALSE);
    videoRecordBin->d->videoRecordQueue = videoQueue;
    
    videoRecordBin->add(videoQueue);
    videoRecordBin->add(colorSpace);
    videoRecordBin->add(videoRate);
    videoRecordBin->add(videoEncoder);
    videoRecordBin->add(videoQueue2);
    videoRecordBin->add(avMuxer);
    videoRecordBin->add(fileSink);
    
    videoRecordBin->add(audioSource);
    videoRecordBin->add(audioQueue);
    videoRecordBin->add(audioConvert);
    videoRecordBin->add(audioEncoder);
    videoRecordBin->add(audioQueue2);
    videoRecordBin->add(audioQueue3);
    

    QGstElement::link(videoQueue,colorSpace,videoRate,videoEncoder,videoQueue2,avMuxer);
    QGstElement::link(avMuxer,fileSink);
    QGstElement::link(audioSource,audioQueue,audioConvert,audioQueue2,audioEncoder,audioQueue3,avMuxer);

    QGstPadPtr pad = videoQueue->getStaticPad("sink");
    QGstGhostPadPtr ghostPad = QGstGhostPad::newGhostPad("sink",pad);
    videoRecordBin->addPad(ghostPad);

    return AVRecordBinPtr(videoRecordBin);
}

AVRecordBin::AVRecordBin()
    : QGstBin("video_recording_bin"), d(new AVRecordBinPrivate)
{
}

AVRecordBin::~AVRecordBin()
{
    delete d;
}

void AVRecordBin::start(QString &fileName)
{
    QByteArray fName = fileName.toUtf8();

    this->setState(QGstElement::Null);
    d->fileSink->setProperty("location", fName);
    this->setState(QGstElement::Playing);
    kDebug() << "Change filename to " << fileName;
}

void AVRecordBin::stop()
{
    //stop video recording by sending eos to record queue
    gst_element_send_event(GST_ELEMENT(d->videoRecordQueue->peekNativeObject()), gst_event_new_eos ());
    this->setState(QGstElement::Null);
}
#include "avrecordbin.moc"