/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "svgbutton.h"
#include <QGraphicsSceneMouseEvent>
#include <KGlobalSettings>
#include <KDebug>
#include <Plasma/Svg>
#include <Plasma/Animator>

SvgButton::SvgButton(QGraphicsWidget *parent)
	: QGraphicsWidget(parent),mSvg(0)
{
    setAcceptsHoverEvents(true);
    mOnHover = false;
    mShowActiveImage = false;
}

SvgButton::~SvgButton()
{
}

void SvgButton::setSvg(const QString &svgFilePath, const QString &elementId)
{
    if(!mSvg)
    {
        mSvg = new Plasma::Svg(this);
        //connect(mSvg,SIGNAL(repaintNeeded()),this,SLOT(svgChanged()));
    }
    
    mSvg->setImagePath(svgFilePath);
    mSvg->setContainsMultipleImages(true);
    mSvgElementId = elementId;
    
    update();
}

void SvgButton::hideActiveImage()
{
    mShowActiveImage = false;
    update();
}

QRectF SvgButton::boundingRect() const
{
    QSize sz=size().toSize();
    return QRectF(0,0,sz.width(),sz.height());
}

void SvgButton::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
    //kDebug()<<"Hover enter";
    mOnHover = true;
    update();
    
    QGraphicsWidget::hoverEnterEvent(event); 
}

void SvgButton::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{ 
    //kDebug()<<"Hover leave";
    mOnHover = false;
    update();
    
    QGraphicsWidget::hoverLeaveEvent(event); 
}

void SvgButton::mouseReleaseEvent(QGraphicsSceneMouseEvent *event) 
{
    kDebug() << "bounding=" << boundingRect() << ",event=" << event->pos();
    if (boundingRect().contains(event->pos())) {
        mShowActiveImage = true;
        emit clicked();
        update();
    } 
}

void SvgButton::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    kDebug()<<"mouse press event";
}

void SvgButton::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    Q_UNUSED(option);
    Q_UNUSED(widget);

    QString element = mSvgElementId;
    if (mSvg){
        if(mShowActiveImage)
            element += "_on";
        if(mOnHover)
            element += "_hover";
        
        mSvg->paint(painter, boundingRect(), element);
    }
}

#include "svgbutton.moc"