/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#include "keren.h"

#include <QGraphicsLinearLayout>
#include <QTimer>
#include <QPainter>
#include <QGraphicsView>
#include <QLabel>
//KDE
#include <KPushButton>
#include <KDebug>
#include <KStandardDirs>
#include <KConfigDialog>
//Plasma
#include <Plasma/PushButton>
#include <Plasma/Frame>
#include <Plasma/IconWidget>
#include <Plasma/Svg>
#include <Plasma/SvgWidget>
#include <Plasma/Label>
#include <gst/base/gstbasesrc.h>
#include "libqtgstreamer/qgstglobal.h"
#include "libqtgstreamer/qgstelement.h"
#include "kerenconfigdialog.h"
#include "previewwidget.h"
#include "infowidget.h"
#include "videogst.h"
#include "videoproxywidget.h"
#include "renderer/videowidget.h"
#include "control/svgbutton.h"

KerenApplet::KerenApplet(QObject *parent, const QVariantList &args)
        : Plasma::PopupApplet(parent, args),
        mCaptureType(KerenApplet::Picture),
        mWidget(0),mVideoGst(0)
{
//    QApplication::setAttribute(Qt::AA_NativeWindows);
    
    QtGstreamer::qGstInit(NULL,NULL);
    setHasConfigurationInterface(true);
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    resize(420, 340);
    setPopupIcon("camera-web");

    mPreviewWidget = new PreviewWidget(this);

    connect(mPreviewWidget, SIGNAL(countDownFinished()), this, SLOT(slotCountDownFinished()));

    mControlFrame = new Plasma::Frame(this);
    mControlFrame->setFrameShadow(Plasma::Frame::Raised);
    
    mPictureButton = new SvgButton(this);
    mVideoButton = new SvgButton(this);
    mPictureButton->setMinimumSize(124,46);
    mVideoButton->setMinimumSize(124,46);
	/*mSvg = new Plasma::Svg();
	mSvg->setImagePath("widgets/keren");
	mSvg->setContainsMultipleImages(true);
	*/
    
	connect(mPictureButton, SIGNAL(clicked()), this, SLOT(slotTakePicture()));
    connect(mVideoButton, SIGNAL(clicked()), this, SLOT(slotTakeVideo()));

    mAlbumName="default";
    mIsRecordingVideo=false;
    mVideoFrameSize = QSize(320,240);
    
    checkHomeDirectoryStucture();
    //TODO: Add device plug unplug signal
}

KerenApplet::~KerenApplet()
{
    if(mVideoGst)
        delete mVideoGst;
}

QGraphicsWidget *KerenApplet::graphicsWidget()
{
    return mWidget;
}

void KerenApplet::constraintsEvent(Plasma::Constraints constraints)
{
    if (constraints & (Plasma::FormFactorConstraint)) {
        if  (this->formFactor() == Plasma::Planar) {
            resumeCapture();
        }
    }
}

void KerenApplet::createConfigurationInterface(KConfigDialog *parent)
{
    mVideoGst->stop();
    mConfigDialog = new KerenConfigDialog(parent);
    mConfigDialog->setVideoFrameSize(mVideoFrameSize);
    mConfigDialog->setVideoFrameRate(mVideoFrameRate);

    kDebug() << "Done set frameSize and framerate";
    parent->addPage(mConfigDialog->audioSettings, i18n("Audio Device"), "audio-input-line");
    parent->addPage(mConfigDialog->videoSettings, i18n("Video Device"), "camera-web");
    
    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(cancelClicked()), this, SLOT(configCancelled()));
    connect(parent, SIGNAL(hidden()), this, SLOT(configCancelled()));
}

void KerenApplet::configAccepted()
{
    mVideoDevicePath = mConfigDialog->videoDevicePath();
    mVideoFrameSize = mConfigDialog->videoFrameSize();
    mVideoFrameRate = mConfigDialog->videoFrameRate();

    KConfigGroup cg = config();
    cg.writeEntry("videodevicepath", mVideoDevicePath);
    cg.writeEntry("videoframesize", mVideoFrameSize);
    cg.writeEntry("videoframerate", mVideoFrameRate);

    emit configNeedsSaving();

    mConfigDialog->stop();
    kDebug() << "update path=" << mVideoDevicePath << ",size=" << mVideoFrameSize << ",fps=" << mVideoFrameRate;
    mVideoGst->setVideoSourceProperties(mVideoDevicePath,mVideoFrameSize,mVideoFrameRate);
    mVideoGst->updatePipeline();
    
    mVideoGst->start();
}

void KerenApplet::configCancelled()
{
    kDebug() << "Config cancelled";
    mConfigDialog->stop();
    mVideoGst->start();
}

void KerenApplet::popupEvent(bool show)
{
    kDebug() << " Popup event="<<show;
    if(show)
        resumeCapture();
    else
        suspendCapture();
}

void KerenApplet::init()
{
    loadConfig();
    createLayout();
    setupPipeline();
}

void KerenApplet::loadConfig()
{
    //query for default value from solid and etc.
    KConfigGroup cg = config();
    
    mVideoDevicePath = cg.readEntry("videodevicepath",QString());
    mVideoFrameSize = cg.readEntry("videoframesize",QSize());
    mVideoFrameRate = cg.readEntry("videoframerate",QSize());

    if (mVideoDevicePath.length() <1) {
        //cannot get from config
        mVideoFrameRate = QSize(30,1);
        mVideoDevicePath = "/dev/video0";
        mVideoFrameSize = QSize(320,240);
    }
}

void KerenApplet::suspendCapture()
{
}

void KerenApplet::resumeCapture()
{
    mVideoGst->start();
}

void KerenApplet::setupPipeline()
{
    mVideoGst = new VideoGst();
    connect(mVideoGst,SIGNAL(photoTaken(QImage *)), this, SLOT(slotScreenshotTaken(QImage *)));
    
    mVideoGst->setVideoSourceProperties(mVideoDevicePath,mVideoFrameSize,mVideoFrameRate);
    mVideoGst->setVideoSink(QtGstreamer::QGstElement::fromGstElement(mPreviewWidget->videoWidget()->nativeWidget()->videoBin()));
    mVideoGst->updatePipeline();
    kDebug() << "linking pipeline";
    mVideoGst->linkPipeline();
    kDebug() << "pipeline linked";
}

void KerenApplet::createLayout()
{
	mPictureButton->setSvg("widgets/keren","snapshot");
	mVideoButton->setSvg("widgets/keren","camcorder");
	
	mPreviewWidget->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
	
    QGraphicsLinearLayout *mainLayout = new QGraphicsLinearLayout(Qt::Vertical);
    QGraphicsLinearLayout *controlLayout = new QGraphicsLinearLayout(Qt::Horizontal);

    QGraphicsWidget *spacerLeft = new QGraphicsWidget();
    spacerLeft->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));

    QGraphicsWidget *spacerRight = new QGraphicsWidget();
    spacerRight->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));

	controlLayout->addItem(spacerLeft);
    controlLayout->addItem(mPictureButton);
    controlLayout->addItem(mVideoButton);
	controlLayout->addItem(spacerRight);
    //controlLayout->setContentsMargins(0, 0, 0, 0);
    mControlFrame->setLayout(controlLayout);

    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->addItem(mPreviewWidget);
    //mainLayout->addItem(controlLayout);
    mainLayout->addItem(mControlFrame);

    if (!mWidget) {
        mWidget = new QGraphicsWidget(this);
        mWidget->setLayout(mainLayout);
        mWidget->setMinimumSize(500,400);
    }
}

void KerenApplet::slotTakePicture()
{
    mCaptureType = KerenApplet::Picture;
    mPreviewWidget->startCountDownTimer(3);
}

void KerenApplet::slotTakeVideo()
{
    if (!mIsRecordingVideo) {
        mIsRecordingVideo = true;
        mCaptureType = KerenApplet::Video;
        mPreviewWidget->startCountDownTimer(3);
    }
    else {
        kDebug() << "Stop recording video";
        mIsRecordingVideo = false;
        mVideoGst->stopVideoRecording();
    }
}

void KerenApplet::slotCountDownFinished()
{
    if (mCaptureType == KerenApplet::Picture) {
        mVideoGst->takeScreenshot();
        mPictureButton->hideActiveImage();
    }
    else {
        kDebug() << "Start recording video";
        QString fileName = createNewCaptureFilename();
        mVideoGst->startVideoRecording(fileName);
    }
}

void KerenApplet::slotScreenshotTaken(QImage *image)
{
    image->save(createNewCaptureFilename());
    delete image;
    kDebug() << "Screenshot taken ";
}

void KerenApplet::checkHomeDirectoryStucture()
{
    mCapturePath = KStandardDirs::locateLocal("data", "keren/captured/", true);
    kDebug() << "Dir=" << mCapturePath;
}

QString KerenApplet::createNewCaptureFilename()
{
    QString fileName;
    QDateTime currentDateTime = QDateTime::currentDateTime();
    
    fileName = mCapturePath + mAlbumName + "/";
    fileName += currentDateTime.toString("ddMMyyyy-hhmmss");
    if (mCaptureType == KerenApplet::Picture) {
        fileName += ".png";
    } 
    else {
        fileName += ".ogg";
    }
    //kDebug() << "File Name=" << fileName;

    return fileName;
}
#include "keren.moc"
