/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#include "photosavebin.h"
#include <QSize>
#include <KDebug>
#include "libqtgstreamer/qgstcaps.h"
#include "libqtgstreamer/qgstpad.h"
#include "libqtgstreamer/qgstghostpad.h"
#include "libqtgstreamer/qgstelementfactory.h"
#include "libqtgstreamer/qgststructure.h"
#include <gst/gst.h>

using namespace QtGstreamer;

class PhotoSaveBinPrivate
{
    public:
        QGstElementPtr capsFilter;
        QGstElementPtr photoSink;
        QSize photoSize;
        uint bufferCallbackId;
        static bool grabPhotoCallback(GstElement *elementSink,GstBuffer *buffer, GstPad *pad, PhotoSaveBin *photoBin);
};

bool PhotoSaveBinPrivate::grabPhotoCallback(GstElement *elementSink,GstBuffer *buffer, GstPad *pad, PhotoSaveBin *photoBin)
{
    unsigned char *data = GST_BUFFER_DATA(buffer);
    int dataSize = GST_BUFFER_SIZE(buffer);
    kDebug() << "grab frame with size=" << dataSize;

    g_signal_handler_disconnect(G_OBJECT(elementSink),photoBin->d->bufferCallbackId);
    QSize photoSize = photoBin->photoSize();
    QImage *image = new QImage(photoSize.width(),photoSize.height(),QImage::Format_RGB888);
    memcpy(image->bits(),data,dataSize);

    emit photoBin->photoTaken(image);
    return true;
}

PhotoSaveBinPtr PhotoSaveBin::createPhotoSaveBin(const QSize &photoSize)
{
    QGstElementPtr queue = QGstElementFactory::make("queue");
    QGstElementPtr colorSpace = QGstElementFactory::make("ffmpegcolorspace");
    QGstElementPtr photoSink = QGstElementFactory::make("fakesink");
    QGstElementPtr capsFilter = QGstElementFactory::make("capsfilter");
    if ( !queue || !colorSpace || !photoSink || !capsFilter ) {
        return PhotoSaveBinPtr();
    }

    PhotoSaveBin *photoCaptureBin = new PhotoSaveBin();
    photoCaptureBin->d->capsFilter = capsFilter;
    photoCaptureBin->d->photoSink = photoSink;
    photoCaptureBin->d->photoSize = photoSize;

    photoCaptureBin->add(photoSink);    
    photoCaptureBin->add(queue);
    photoCaptureBin->add(colorSpace);
    photoCaptureBin->add(capsFilter);

    photoSink->setProperty("signal-handoffs",TRUE);

    QGstElement::link(queue,colorSpace,capsFilter,photoSink);

    QGstCapsPtr caps = QGstCaps::newEmpty();
    QGstStructure rgbStruct("video/x-raw-rgb");
    rgbStruct.setValue("width", photoSize.width());
    rgbStruct.setValue("height", photoSize.height());

    caps->makeWritable();
    caps->appendStructure(rgbStruct);

    capsFilter->setProperty("caps",caps);

    QGstPadPtr pad = queue->getStaticPad("sink");
    QGstGhostPadPtr ghostPad = QtGstreamer::QGstGhostPad::newGhostPad("sink",pad);

    photoCaptureBin->addPad(ghostPad);
    return PhotoSaveBinPtr(photoCaptureBin);
}

PhotoSaveBin::PhotoSaveBin()
    : QGstBin("photo_capture_bin"), d(new PhotoSaveBinPrivate)
{
}

PhotoSaveBin::~PhotoSaveBin()
{
    delete d;
}

QSize &PhotoSaveBin::photoSize() const
{
    return d->photoSize;
}

void PhotoSaveBin::updatePhotoSize(const QSize &photoSize)
{
    d->photoSize = photoSize;

    QGstCapsPtr caps = QGstCaps::newEmpty();
    QGstStructure rgbStruct("video/x-raw-rgb");
    rgbStruct.setValue("width", d->photoSize.width());
    rgbStruct.setValue("height", d->photoSize.height());

    caps->makeWritable();
    caps->appendStructure(rgbStruct);

    d->capsFilter->setProperty("caps",caps);
}

void PhotoSaveBin::takePhoto()
{
    d->bufferCallbackId = g_signal_connect(
            G_OBJECT(d->photoSink->peekNativeObject()), "handoff",G_CALLBACK(d->grabPhotoCallback), this);
}
#include "photosavebin.moc"