/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#ifndef VIDEO_GST_H
#define VIDEO_GST_H
#include <QImage>
#include <gst/gst.h>
#include "libqtgstreamer/qgstdeclarations.h"

namespace Gstreamer
{
    class VideoDevice;
}

class PhotoSaveBin;
typedef QSharedPointer<PhotoSaveBin> PhotoSaveBinPtr;
class AVRecordBin;
typedef QSharedPointer<AVRecordBin> AVRecordBinPtr;
class VideoEffectBin;
typedef QSharedPointer<VideoEffectBin> VideoEffectBinPtr;
class VideoCaptureBin;
typedef QSharedPointer<VideoCaptureBin> VideoCaptureBinPtr;

class VideoGst : public QObject
{
    Q_OBJECT
public:
    VideoGst(QObject *parent=0);
    ~VideoGst();

    bool isRunning();
    bool isRecording();
    void linkPipeline();
    void updatePipeline();
    void setVideoSink(const QtGstreamer::QGstElementPtr &videoSink);
    void setVideoSourceProperties(QString &devicePath,QSize frameSize,QSize frameRate);
    void start();
    void stop();
    void startVideoRecording(QString &fileName);
    void stopVideoRecording(bool fromCallback = false);
    void takeScreenshot();
    QSize frameSize() { return mFrameSize; }

signals:
    void photoTaken(QImage *image);

private slots:
    void slotScreenshotTaken(QImage *image);

private:
    void setupPipeline();
    void updateVideoSource();
    void updatePhotoSaveBin();

    static gboolean busCall(GstBus *bus, GstMessage *msg, gpointer data);

private:
    bool mIsRunning;
    bool mIsRecording;
    QString mDevicePath;
    QSize mFrameRate;
    QSize mFrameSize;

    PhotoSaveBinPtr mPhotoSaveBin;
    AVRecordBinPtr mVideoRecordingBin;
    VideoEffectBinPtr mVideoEffectBin;
    VideoCaptureBinPtr mVideoCaptureBin;

    QtGstreamer::QGstPipelinePtr mPipeline;
    QtGstreamer::QGstElementPtr mVideoSinkPtr;
};

#endif