/*
 * This file is part of the KDE project
 * Copyright (C) 2009 Eva Brucherseifer <eva.brucherseifer@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef KILLSWITCHMANAGER_H
#define KILLSWITCHMANAGER_H

#include <QtCore/QObject>
#include <QtCore/QVariant>
#include <QtCore/QStringList>

class KillSwitchManagerPrivate;


class KillSwitchManager : public QObject
{
    Q_OBJECT

public:
    KillSwitchManager(QObject *parent);
    virtual ~KillSwitchManager();

    virtual bool deviceExists(const QString &udi);
    virtual QStringList devicesFromQuery(const QString &parentUdi);
    virtual QObject *createDevice(const QString &udi);

Q_SIGNALS:
    /**
     * This signal is emitted when a new device appear in the system.
     *
     * @param udi the new device identifier
     */
    void deviceAdded(const QString &udi);

    /**
     * This signal is emitted when a device disappear from the system.
     *
     * @param udi the old device identifier
     */
     void deviceRemoved(const QString &udi);

private Q_SLOTS:
    void slotDeviceAdded(const QString &udi);
    void slotDeviceRemoved(const QString &udi);

private:
    QStringList findDeviceStringMatch(const QString &key, const QString &value);
    QStringList findDeviceByDeviceInterface(const QStringList &cap_list);

    KillSwitchManagerPrivate *d;
};

#endif // KILLSWITCHMANAGER_H
