/***************************************************************************
 *   Copyright (C) 2010 Reza Fatahilah Shah <rshah0385@kireihana.com>      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "knewstuffplasmoid.h"
#include <QGraphicsLinearLayout>
#include <QStandardItemModel>
#include <QStandardItem>

#include <KLineEdit>
//#include <knewstuff3/downloadmanager.h>
#include <KNotification>
#include <KIcon>
#include <KIconLoader>
#include <KDebug>

#include <plasma/tooltipmanager.h>
#include <Plasma/IconWidget>
#include <Plasma/PushButton>
#include <Plasma/LineEdit>
#include <Plasma/DataEngine>
#include <Plasma/DataEngineManager>
#include <KToolInvocation>
#include <KRun>
//#include "listview.h"


KNewStuffPlasmoid::KNewStuffPlasmoid(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args)
    ,mUpdateTimer(0),mUpdateInterval(10000)
    //,mNewStuffManager(0)
{
}

KNewStuffPlasmoid::~KNewStuffPlasmoid()
{
    //what should be deleted here?
}

void KNewStuffPlasmoid::init()
{
    //setPopupIcon("get-hot-new-stuff");
    
    createUi();
    
    mUpdateTimer = new QTimer(this);
    mUpdateTimer->setInterval(mUpdateInterval);
    connect(mUpdateTimer, SIGNAL(timeout()), this, SLOT(slotUpdateTimer()));
    mUpdateTimer->start();
}

void KNewStuffPlasmoid::createUi()
{
    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(this);
    layout->setContentsMargins(0,0,0,0);
    layout->setSpacing(0);

    mKNewstuffIcon = new Plasma::IconWidget(KIcon("get-hot-new-stuff"),"");
    layout->addItem(mKNewstuffIcon);
    setLayout(layout);
    
    QString tooltipText = "";
    
    Plasma::ToolTipContent toolTipContent(i18n("KNewstuff Update Checker"),
                                tooltipText,KIcon("get-hot-new-stuff"));
    Plasma::ToolTipManager::self()->setContent(mKNewstuffIcon, toolTipContent);
    setMaximumSize(64,64);
}

int updateCount=1;
void KNewStuffPlasmoid::slotUpdateTimer()
{
    QString tooltipText = i18np("You have 1 update available","You have %1 updates available",updateCount);
    Plasma::ToolTipContent toolTipContent(i18n("KNewstuff Update Checker"),
                                tooltipText,KIcon("get-hot-new-stuff"));
    Plasma::ToolTipManager::self()->setContent(mKNewstuffIcon, toolTipContent);
    
    kDebug() << "check for update";
    kDebug() << "send notification";
    QString message = i18ncp("Notification when updates available","You have 1 update","You have %1 updates", updateCount);
    sendNotification(message);
    updateCount+=2;
}

/*void KNewStuffPlasmoid::slotCheckUpdate()
{
    QString fileName = mKnsrcTextInput->text();
    kDebug() <<"load=" << fileName;
    newCheckingSession(fileName);
    mNewStuffManager->checkForUpdates();
}

void KNewStuffPlasmoid::slotCheckNewest()
{
    QString fileName = mKnsrcTextInput->text();
    kDebug() <<"load=" << fileName;
    newCheckingSession(fileName);
    
    mNewStuffManager->setSearchOrder(KNS3::DownloadManager::Newest);
    mNewStuffManager->search();
}*/
/*
void KNewStuffPlasmoid::newCheckingSession(QString &fileName)
{
    //finish old session if any
    if (mNewStuffManager) {
        disconnect(mNewStuffManager, SIGNAL(searchResult(KNS3::Entry::List)), this, SLOT(slotUpdatesAvailable(KNS3::Entry::List)));
        
        delete mNewStuffManager;
    }
    if (!mEntryModel) {
        mEntryModel = new QStandardItemModel();
    }
    mEntryModel->clear();
    setBusy(true);
    mNewStuffManager = new KNS3::DownloadManager(fileName, this);
    connect(mNewStuffManager, SIGNAL(searchResult(KNS3::Entry::List)), this, SLOT(slotUpdatesAvailable(KNS3::Entry::List)));    
}

void KNewStuffPlasmoid::slotUpdatesAvailable(const KNS3::Entry::List& updates)
{
    setBusy(false);
    mUpdateableEntries = updates;
    kDebug() << "got updates=" << mUpdateableEntries.size();
    
    if (mUpdateableEntries.size() <1) {
        return;
    }
    
    QString message = i18nc("Notification when updates available","You have %1 updates", mUpdateableEntries.size());
    
    //sendNotification(message);
    
    QStandardItem *item;
    foreach (const KNS3::Entry& entry, updates) {
        item = new QStandardItem(entry.name());
        mEntryModel->appendRow(item);
    }
    
    mEntriesListView->setModel(mEntryModel);
}
*/
void KNewStuffPlasmoid::sendNotification(QString &message)
{
    QPixmap pix = KIcon("get-hot-new-stuff").pixmap(KIconLoader::SizeMedium, KIconLoader::SizeMedium);
    
    KNotification *notify = new KNotification("updatesavailable");    
    connect(notify,SIGNAL(action1Activated() ) , this, SLOT( slotOpenKNewstuffDialog()));
    notify->setComponentData(KComponentData("plasma_knewstuff_plasmoid"));
    //notify->setText(i18nc("Notification when the pastebin applet has copied the URL to the clipboard",
                          //"%1 has been copied to your clipboard", url));
    notify->setText(message);
    notify->setActions(QStringList(i18n("Open KNewStuff Dialog")));
    //notify->setPixmap(pix);
    
    notify->sendEvent();
}

void KNewStuffPlasmoid::slotOpenKNewstuffDialog()
{
    kDebug() << "open new stuff";
    QString command="khotnewstuff4 wallpaper.knsrc";
    KRun::runCommand(command,"khotnewstuff4","khotnewstuff4",0);
}
#include "knewstuffplasmoid.moc"
