/*
    This file is part of the Program Menu plasmoid, part of the KDE project.

    Copyright (c) 2002 Siegfried Nijssen <snijssen@liacs.nl>
    Copyright (c) 2003 Lubos Lunak <l.lunak@suse.cz>
    Copyright 2008 Friedrich W. H. Kossebau <kossebau@kde.org>

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
    AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
    AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
    CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include "menubarembedcontainer.h"
#include "menubarembedcontainer.moc"
// explicit moc needed, because with automoc X11 headers could happen to be not last

// Qt
#include <QtGui/QX11Info>
// C
#include <stdio.h>


static Atom selectionAtom = None;
static Atom messageTypeAtom = None;

static void initAtoms()
{
    char ownerName[ 100 ];
    sprintf( ownerName, "_KDE_TOPMENU_OWNER_S%d", DefaultScreen(QX11Info::display()) );
    char minSizeName[] = "_KDE_TOPMENU_MINSIZE";
    char *names[ 2 ] = { ownerName, minSizeName };

    Atom atoms[ 2 ];
    XInternAtoms( QX11Info::display(), names, 2, False, atoms );

    selectionAtom = atoms[ 0 ];
    messageTypeAtom = atoms[ 1 ];
}

Atom MenuBarEmbedContainer::makeSelectionAtom()
{
    if( selectionAtom == None )
        initAtoms();
    return selectionAtom;
}


MenuBarEmbedContainer::MenuBarEmbedContainer( WId mainWindowId )
 : QX11EmbedContainer( /*parent*/0 ),
   mMainWindowId( mainWindowId )
{
    connect( this, SIGNAL(clientClosed()), SLOT(onClientClosed()) );
    connect( this, SIGNAL(error( QX11EmbedContainer::Error )), SLOT(onError( QX11EmbedContainer::Error )) );
}

bool MenuBarEmbedContainer::x11Event( XEvent *event )
{
    if( event->type == ConfigureRequest
        && event->xconfigurerequest.window == clientWinId()
        && event->xconfigurerequest.value_mask & (CWWidth|CWHeight) )
    {
        const XConfigureRequestEvent &configureRequestEvent = event->xconfigurerequest;
        // resize when the embedded window resizes (still obey min size)
//      kdDebug() << "RES:" << clientWinId() << ":" << configureRequestEvent.width << ":" << configureRequestEvent.height << endl;
        if( configureRequestEvent.width != width() || configureRequestEvent.height != height())
        {
            resize( configureRequestEvent.width, configureRequestEvent.height );
            emit sizeChanged( this );
        }
        sendSyntheticConfigureNotifyEvent();
//        int x, y;
//        unsigned int w, h, d, b;
//        Window root;
//        XGetGeometry( QX11Info::display(), clientWinId(), &root, &x, &y, &w, &h, &b, &d );
//        kdDebug() << "RES3:" << width() << ":" << height() << ":" << w << ":" << h << endl;
        return true;
    }

    return QX11EmbedContainer::x11Event( event );
}


void MenuBarEmbedContainer::sendSyntheticConfigureNotifyEvent() const
{
    if( clientWinId() != None )
    {
        const QPoint globalPos = mapToGlobal( QPoint(0,0) );

        XConfigureEvent configureEvent;
        memset( &configureEvent, 0, sizeof(configureEvent) );
        configureEvent.type = ConfigureNotify;
        configureEvent.display = QX11Info::display();
        configureEvent.send_event = True;
        configureEvent.event = clientWinId();
        configureEvent.window = winId();
        configureEvent.x = globalPos.x();
        configureEvent.y = globalPos.y();
        configureEvent.width = width();
        configureEvent.height = height();
        configureEvent.border_width = 0;
        configureEvent.above = None;
        configureEvent.override_redirect = 0;

        XSendEvent( QX11Info::display(), configureEvent.event, true, StructureNotifyMask, (XEvent*)&configureEvent );
    }
}


void MenuBarEmbedContainer::setMinimumSize( int width, int height )
{
    QX11EmbedContainer::setMinimumSize( width, height );
    // tell the menubar also the allowed minimum size
    // the plasmoid won't allow resizing to smaller size
    if( clientWinId() != None )
    {
        XEvent event;
        event.xclient.display = QX11Info::display();
        event.xclient.type = ClientMessage;
        event.xclient.window = clientWinId();
        Q_ASSERT( messageTypeAtom != None );
        event.xclient.message_type = messageTypeAtom;
        event.xclient.format = 32;
        event.xclient.data.l[0] = QX11Info::appTime();
        event.xclient.data.l[1] = minimumWidth();
        event.xclient.data.l[2] = minimumHeight();
        event.xclient.data.l[3] = 0;
        event.xclient.data.l[4] = 0;

        XSendEvent( QX11Info::display(), clientWinId(), False, NoEventMask, &event );
    }
}

void MenuBarEmbedContainer::onClientClosed()
{
    emit menuLost( this );
}
void MenuBarEmbedContainer::onError( QX11EmbedContainer::Error error )
{
kDebug() << "what a pity, embedding error"<<error;
    emit menuLost( this );
}

MenuBarEmbedContainer::~MenuBarEmbedContainer() {}
