/***********************************************************************************
* Mini Player: Advanced media player for Plasma.
* Copyright (C) 2008 - 2009 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#ifndef MINIPLAYERPLAYLISTMODEL_HEADER
#define MINIPLAYERPLAYLISTMODEL_HEADER

#include <QHash>
#include <QVariant>
#include <QMimeData>
#include <QAbstractTableModel>

#include <KUrl>

class MiniPlayerApplet;

class MiniPlayerPlayListModel : public QAbstractTableModel
{
    Q_OBJECT

    public:
        MiniPlayerPlayListModel(MiniPlayerApplet *parent);

        void addTracks(KUrl::List items, int index = 0);
        void removeTracks(KUrl::List items);
        void clearPlayList();
        void sort(int column, Qt::SortOrder order);
        KUrl::List getTracks();
        QVariant data(const QModelIndex &index, int role) const;
        QVariant headerData(int section, Qt::Orientation orientation, int role) const;
        QMimeData* mimeData(const QModelIndexList &indexes) const;
        Qt::ItemFlags flags(const QModelIndex &index) const;
        Qt::DropActions supportedDropActions() const;
        int columnCount(const QModelIndex &index) const;
        int rowCount(const QModelIndex &index) const;
        int currentTrack() const;
        bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole);
        bool dropMimeData(const QMimeData *mimeData, Qt::DropAction action, int row, int column, const QModelIndex &index);
        bool insertRows(int row, int count, const QModelIndex &index = QModelIndex());
        bool removeRows(int row, int count, const QModelIndex &index = QModelIndex());

    public slots:
        void setCurrentTrack(int track);

    private:
        MiniPlayerApplet *m_player;
        KUrl::List m_urls;
        QStringList mimeTypes() const;
        int m_currentTrack;

    signals:
        void needsSaving();
        void itemChanged(QModelIndex index);
};

#endif
