/*
 *   Copyright (C) 2007 Percy Leonhardt <percy@eris23.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef NETWORK_H
#define NETWORK_H

#include <QFont>
#include <QColor>
#include <QString>
#include <QVector>

#include <plasma/applet.h>
#include <plasma/dataengine.h>

#include "ui_config.h"

class KDialog;

namespace Plasma
{
    class Svg;
    class SignalPlotter;
}

class Network : public Plasma::Applet
{
    Q_OBJECT
public:
    Network(QObject *parent, const QVariantList &args);
    ~Network();

    void init();
    QSizeF contentSizeHint() const;
    void paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect);
    Qt::Orientations expandingDirections() const;

public slots:
    void dataUpdated(const QString &source, const Plasma::DataEngine::Data &data);
    void showConfigurationInterface();

protected:
    bool sceneEvent(QEvent *event);
    void constraintsEvent(Plasma::Constraints constraints);

protected slots:
    void configAccepted();

private:
    void updateSize();
    void configurePlotter();
    void paintTrafficIcon(QPainter *p, const QRect &geometry);

    /**
     * Helper class to store the width of a string
     */
    class Label
    {
    public:
        Label();

        void setText(const QString &text);
        void setParent(const Network *parent);
        void invalidate();
        int width() const;
        QString text() const;

    private:
        int m_textWidth;
        QString m_text;
        const Network *m_parent;
    };

    /**
     * Helper class to create, store and manage labels
     */
    class Container
    {
    public:
        Container(const Network *parent);
        ~Container();

        void invalidateAll();
        Network::Label& getLabel(int row, int col);
        int getNumRows();
        int getRowWidth(int row);

    private:
        typedef QVector<Label> LabelRow;
        typedef QVector<LabelRow> LabelGrid;
        LabelGrid m_grid;

        const Network *m_parent;
    };

    enum DetailLevel {
        MinimumDetails = 0,
        LowDetails,
        MediumDetails,
        HighDetails,
        AllDetails
    };
    DetailLevel m_detailLevel;

    int m_margin;
    int m_spacing;
    int m_rxPackets;
    int m_txPackets;
    bool m_isWireless;
    bool m_connectionActive;
    QFont m_textFont;
    QSizeF m_size;
    QColor m_textColor;
    QColor m_incomingColor;
    QColor m_outgoingColor;
    QString m_rxState;
    QString m_txState;
    QString m_quality;
    KDialog *m_dialog;
    QFontMetrics m_fontMetrics;

    Container m_container;

    Ui::config ui;
    Plasma::Svg *m_traffic;
    Plasma::SignalPlotter *m_plotter;
};

K_EXPORT_PLASMA_APPLET(network, Network)

#endif
