/***************************************************************************
 *   Copyright (C) 2008 by Matej Svejda <mata@aw-modell.at>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "peachy_layout.h"

#include "peachy_icon.h"
#include "peachy_applet.h"

#include <QRectF>
#include <QSizeF>
#include <QGraphicsLayoutItem>

#include <KDebug>

PeachyLayout::PeachyLayout(PeachyApplet* parent)
	: QGraphicsLinearLayout(parent),
	m_borderSpacing(5),
	m_maximumIconSize(0)
{

}

PeachyLayout::~PeachyLayout()
{

}

QGraphicsLayoutItem* PeachyLayout::itemAt(int i) const
{
	return m_icons[i];
}

int PeachyLayout::count() const
{
	return m_icons.count();
}

void PeachyLayout::removeAt(int i)
{
	m_icons.removeAt(i);
}

void PeachyLayout::add(PeachyIcon* icon)
{
	icon->setMaximumIconSize(m_maximumIconSize);
	m_icons.append(icon);
}

void PeachyLayout::remove(PeachyIcon* icon)
{
	m_icons.removeAll(icon);
}

void PeachyLayout::clear()
{
	m_icons.clear();
}

void PeachyLayout::setGeometry(const QRectF& rect)
{
	// Determine how much space there is for an icon
	//TODO: Maybe check width too
	qreal maximumIconSize = rect.height() - 2 * m_borderSpacing;
	if(maximumIconSize < 0) {
		maximumIconSize = 0;
	}
	//kDebug() << rect.width();
	qreal itemWidth = 0;
	if(m_maximumIconSize != maximumIconSize) {
		// Maximum size has changed so tell the icons so they can load propper icons.
		foreach(PeachyIcon* icon, m_icons) {
			icon->setMaximumIconSize(maximumIconSize);
		}
		m_maximumIconSize = maximumIconSize;
	}
	// Calculate how much width there is so the icons can be centered.
	foreach(PeachyIcon* icon, m_icons) {
		itemWidth += icon->size();
	}
	qreal position = (rect.width() - itemWidth) / 2;
	if(position < 0) {
		position = 0;
	}
	// Center them...
	foreach(PeachyIcon* icon, m_icons) {
		icon->setGeometry(QRectF(position, m_borderSpacing + rect.height() - icon->size(), icon->size(), icon->size()));
		position += icon->size();
	}
}

void PeachyLayout::setBorderSpacing(qreal spacing)
{
	m_borderSpacing = spacing;
	invalidate();
}

QSizeF PeachyLayout::sizeHint(Qt::SizeHint hint, const QSizeF& size) const
{
	return QSizeF(-1, -1);
}
