/***************************************************************************
 *   Copyright (C) 2009 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "gamelabel.h"
#include "game.h"

// Qt
#include <QGraphicsSceneWheelEvent>

// KDE
#include <Plasma/Theme>


GameLabel::GameLabel(QGraphicsWidget *parent, Game *game, const GameLabel::Type &type)
    : Plasma::Label(parent),
    m_game(game), m_type(type)
{
    
    setAcceptDrops(false);
    switch (m_type) {
        case GameLabel::Title: {
            setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
            break;
        }
        case GameLabel::Description: {
            setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
            break;
        }
    } 
    
    nativeWidget()->setWordWrap(false);
    
    connect(m_game, SIGNAL(changed()), this, SLOT(gameChanged()));
    gameChanged();
    
}


GameLabel::~GameLabel()
{



}


void GameLabel::gameChanged()
{
    
    switch (m_type) {
        case GameLabel::Title: setText(m_game->title()); break;
        case GameLabel::Description: setText(m_game->description()); break;
    }
    updateGeometry();

}


QSizeF GameLabel::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
    
    Q_UNUSED(constraint);
    
    QSizeF hint;
    QFontMetricsF fm(Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont));
    
    hint.setHeight(fm.height());
    QString text;
    switch (m_type) {
        case GameLabel::Title: text = m_game->title(); break;
        case GameLabel::Description: text = m_game->description(); break;
    }
    hint.setWidth(fm.width(text)+2);
    
    QString firstWord = text.section(' ', 0, 0);
    QSizeF min = QSizeF(fm.width(firstWord), fm.height());
    
    switch (which) {
        case Qt::MinimumSize: return min;
        case Qt::PreferredSize: return hint;
        case Qt::MinimumDescent: return min;
        case Qt::MaximumSize: return hint;
        case Qt::NSizeHints: return min;
        default: return min;
    }
    
}


void GameLabel::wheelEvent(QGraphicsSceneWheelEvent *event)
{

    event->ignore();

}


#include "gamelabel.moc"
