/***************************************************************************
 *   Copyright (C) 2010 by Tomaz Canabrava <tcanabrava@kde.org>                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "recentappdocuments.h"
#include "iconwidget.h"

#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <QFileInfo>

#include <QGraphicsLinearLayout>
#include <QGraphicsScene>
#include <QGraphicsSceneHoverEvent>
#include <QGraphicsLayoutItem>

#include <KLocale>
#include <KAction>
#include <KMimeType>

#include <Plasma/Label>

#include <Plasma/ScrollWidget>
#include <Plasma/ItemBackground>

RecentAppDocumentsApplet::RecentAppDocumentsApplet(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args)
{
    setBackgroundHints(DefaultBackground);
    setHasConfigurationInterface(false);
    m_showInfo = false;
 
    QGraphicsLinearLayout *mainLayout = new QGraphicsLinearLayout();
    
    m_contentLayout = new QGraphicsLinearLayout();
    
    mainLayout->setOrientation(Qt::Vertical);
    m_contentLayout->setOrientation(Qt::Vertical);
    Plasma::Label *label = new Plasma::Label(this);
    label->setText("Open Recent");
    mainLayout->addItem(label);
    
    QGraphicsWidget *mainWidget = new QGraphicsWidget;
    Plasma::ScrollWidget * scrollWidget = new Plasma::ScrollWidget(this);
    scrollWidget->setWidget(mainWidget);
    scrollWidget->setOverflowBordersVisible(true);
    mainLayout->addItem(scrollWidget);
    setLayout(mainLayout);
    
    mainWidget->setLayout(m_contentLayout);
    m_background = new Plasma::ItemBackground(this);
    m_hovered = 0;
}


RecentAppDocumentsApplet::~RecentAppDocumentsApplet()
{

}

void RecentAppDocumentsApplet::init()
{
    dataEngine("recentappdocuments")->connectSource(KGlobal::mainComponent().aboutData()->programName(), this);
}

void RecentAppDocumentsApplet::recentDocumentSelectedEmiter()
{
    emit recentDocumentSelected(static_cast<KAction*>(sender())->data().toString());
}

void RecentAppDocumentsApplet::dataUpdated (const QString &sourceName, const Plasma::DataEngine::Data &data){
    if (sourceName != KGlobal::mainComponent().aboutData()->programName())
        return;

    Q_FOREACH(IconWidget *icon, iconList){
        icon->deleteLater();
    }
    iconList.clear();
    
    Q_FOREACH(const QString& key, data.keys()){
        QFileInfo fileInfo(key);
        QString s = fileInfo.baseName();
        if(s.length() > 30){
            s =  s.rightJustified(30, '.', true);
            s += QString("(...)");
        }
        s += QString(".") + fileInfo.suffix();
        KAction *action = new KAction(KIcon(KMimeType::iconNameForUrl(key)), s, this);
        action->setData(key);
        action->setToolTip(key);

        IconWidget  *icon = new IconWidget(this);
        icon->setAction(action);
        icon->setMaximumIconSize(QSize(16,16));
        icon->setOrientation(Qt::Horizontal);
        icon->setBackgroundItem( m_background );
        
        if(m_showInfo)
            icon->setInfoText(key);

        connect(action, SIGNAL(triggered()), this, SLOT(recentDocumentSelectedEmiter()));

        m_contentLayout->addItem(icon);
        iconList << icon;
    }
    
    KAction *action = new KAction(KIcon("document-open"), i18n("Chose other"), this);
    connect(action, SIGNAL(triggered()), this, SIGNAL(openFileDialog()));
    IconWidget *icon = new IconWidget(this);
    icon->setAction(action);
    icon->setMaximumIconSize(QSize(16,16));
    icon->setOrientation(Qt::Horizontal);
    icon->setBackgroundItem( m_background );
    iconList << icon;
    m_contentLayout->addItem(icon);
}

K_EXPORT_PLASMA_APPLET(recentappdocuments, RecentAppDocumentsApplet);

#include "recentappdocuments.moc"
