/***************************************************************************
 *   Copyright 2008 by Beat Wolf <asraniel@fryx.ch>                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "configuration.h"

#include <QGridLayout>
#include <QPushButton>
#include <QLabel>
#include <QCheckBox>

#include <KLocale>

Configuration::Configuration(const KConfigGroup &con, QList<struct service> *services){
    m_config = con;
    base_services = services;

    QGridLayout *layout = new QGridLayout();
    //TODO: find out more about i18n and kde, and adapt the interace
    QLabel *nameLabel = new QLabel(i18n("Server name"));
    name = new QLineEdit(m_config.readEntry("name"));
    QLabel *addressLabel = new QLabel(i18n("Host address"));
    hostAddress = new QLineEdit(m_config.readEntry("address"));
    QLabel *usernameLabel = new QLabel(i18n("Ssh User name"));
    userName = new QLineEdit(m_config.readEntry("username"));

    QPushButton *ok = new QPushButton(i18n("Ok"));
    connect(ok, SIGNAL(clicked()), this, SLOT(ok()));
    QPushButton *cancel = new QPushButton(i18n("Cancel"));
    connect(cancel, SIGNAL(clicked()), this, SLOT(cancel()));

    //create kio list
    QWidget *serviceList = new QWidget();
    QGridLayout *layout_services = new QGridLayout();
    m_service_choices = new QButtonGroup();
    m_service_choices->setExclusive(false);

    QStringList pre_selected = m_config.readEntry("services", QStringList());
    QLabel *kioLabel = new QLabel(i18n("KIO services"));
    
    int place = 0;
    for(int i = 0; i < base_services->size(); i++){
        if(base_services->at(i).type == SERVICE){
          QCheckBox *temp = new QCheckBox(base_services->at(i).name);

          if(pre_selected.contains(base_services->at(i).protocol)){
              temp->setChecked(true);
          }

          m_service_choices->addButton(temp);
          layout_services->addWidget(temp,place,0);
          place++;
        }
    }

    serviceList->setLayout(layout_services);

    QLabel *shellLabel = new QLabel(i18n("Remote shell"));

    //WARNING: Code duplication--start
    QWidget *shellList = new QWidget();
    QGridLayout *layout_shells = new QGridLayout();
    place = 0;
    for(int i = 0; i < base_services->size(); i++){
        if(base_services->at(i).type == SHELL){
          QCheckBox *temp = new QCheckBox(base_services->at(i).name);

          if(pre_selected.contains(base_services->at(i).protocol)){
              temp->setChecked(true);
          }

          m_service_choices->addButton(temp);
          layout_shells->addWidget(temp,place,0);
          place++;
        }
    }

    // WARNING: hardcoded ssh index! (by FiNeX)
    userName->setEnabled(m_service_choices->buttons().at(4)->isChecked());
    connect (m_service_choices->buttons().at(4), SIGNAL(toggled(bool)), userName, SLOT(setEnabled(bool)) );

    shellList->setLayout(layout_shells);
    //WARNING: Code duplication--end

    //create dialog layout
    layout->addWidget(nameLabel,0,0,1,2);
    layout->addWidget(name,1,0,1,2);
    
    layout->addWidget(addressLabel,2,0,1,2);
    layout->addWidget(hostAddress,3,0,1,2);
    layout->addWidget(kioLabel,4,0);
    layout->addWidget(serviceList,5,0,1,2);
    layout->addWidget(shellLabel,6,0);
    layout->addWidget(shellList,7,0,1,2);

    layout->addWidget(usernameLabel,8,0,1,2);
    layout->addWidget(userName,9,0,1,2);
    
    layout->addWidget(ok,10,0);
    layout->addWidget(cancel,10,1);

    setLayout(layout);  

    show();
}

Configuration::~Configuration(){
    delete name;
    delete hostAddress;
    delete m_service_choices;
}

void Configuration::ok(){
    m_config.writeEntry("name",name->text());
    m_config.writeEntry("address",hostAddress->text());
    m_config.writeEntry("username",userName->text());

    QStringList services_selected;

    for(int i = 0; i < m_service_choices->buttons().size(); i++){
        if(m_service_choices->buttons().at(i)->isChecked())
          services_selected << base_services->at(i).protocol;
    }

    m_config.writeEntry("services",services_selected);

    accept();
}

void Configuration::cancel(){
    reject();
}

#include "configuration.moc"
