#include "stockitem.h"
#include <QPainter>
#include <klocalizedstring.h>

StockItem::StockItem(QGraphicsItem *parent)
    : SvgLayoutItem(parent), m_sName(i18n("Waiting...")), m_dblValue(0.0)
{
    m_minSize = QSizeF(200.0, 25.0);
    setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding));
    setImagePath("widgets/stock_bar_neutral");
}

StockItem::~StockItem()
{
}

void StockItem::setId(QString sId)
{
    m_sId = sId;
}

QString StockItem::id() const
{
    return m_sId;
}

void StockItem::setName(QString sName)
{
    m_sName = sName;
}

QString StockItem::name() const
{
    return m_sName;
}

void StockItem::setValue(qreal dblValue)
{
    m_dblValue = dblValue;
    QString sTmp = m_sName + QString::number(m_dblValue);
    QFont fontNormal;
    fontNormal.setStyleHint(QFont::SansSerif);
    fontNormal.setPointSize(12);
    fontNormal.setWeight(QFont::Bold);
    QFontMetrics m(fontNormal);

    QRect r = m.boundingRect(sTmp);
    m_minSize = QSizeF(r.width()+10, r.height()+10);
    updateGeometry();
}

qreal StockItem::value() const
{
    return m_dblValue;
}

void StockItem::setChange(qreal dblChange)
{
    m_dblChange = dblChange;
}

qreal StockItem::change() const
{
    return m_dblChange;
}

void StockItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    QRectF rTextRect = boundingRect().adjusted(5, 5, -5, -5);
    QFont fontNormal;
    fontNormal.setStyleHint(QFont::SansSerif);
    fontNormal.setPointSize(10);
    fontNormal.setWeight(QFont::Bold);

    SvgLayoutItem::paint(painter, option, widget);
    painter->setPen(QPen(Qt::white));
    painter->setFont(fontNormal);
    painter->drawText(rTextRect, Qt::AlignLeft | Qt::AlignVCenter, m_sName);
    painter->drawText(rTextRect, Qt::AlignRight | Qt::AlignVCenter, QString::number(m_dblValue));
}

QSizeF StockItem::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
    switch(which)
    {
    case Qt::MinimumSize:
        return m_minSize;
    case Qt::PreferredSize:
        return m_minSize;
    default:
        return QSizeF();
    }
}

