/***************************************************************************
 *   Copyright (C) 2008 by Gerhard Gappmeier   *
 *   gerhard.gappmeier@ascolab.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "stockwidget.h"
#include "stockitem.h"
#include "trenditem.h"
#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <QGraphicsGridLayout>

#include <plasma/svg.h>
#include <plasma/theme.h>

#define STOCK_HEIGHT     30
#define STOCK_WIDTH     200
#define STOCK_STAT_WIDTH 60
#define STOCK_SPACING     1

StockWidget::StockWidget(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args)
{
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    setMinimumSize(100, 100);
    resize(350, 250);

    m_pGridLayout = new QGraphicsGridLayout();
    setLayout(m_pGridLayout);
}

StockWidget::~StockWidget()
{
    if (hasFailedToLaunch()) {
        // Do some cleanup here
    } else {
        // Save settings
    }
}

void StockWidget::init()
{
    m_pStocksEngine = dataEngine("stocks");
    if (m_pStocksEngine)
    {
        kDebug() << "Loaded stocks dataEngine.";
        // connect test source
        addStock("^GDAXI", "DAX Index");
        addStock("^DJI", "DOW JONES INDUSTRIAL AVERAGE IN");
        addStock("^NYA", "NYSE COMPOSITE INDEX");
        addStock("GB0030932676EUR", "M&G Global Basics A EUR Acc");
        addStock("SI", "Siemens AG");
        addStock("GOOG", "Google Inc.");
        
        connect(m_pStocksEngine, SIGNAL(sourceAdded(const QString&)), this, SLOT(sltNewSource(const QString&)));
        connect(m_pStocksEngine, SIGNAL(sourceRemoved(const QString&)), this, SLOT(sltSourceRemoved(const QString&)));
    }
    else
    {
        kDebug() << "Could not load stocks dataEngine.";
    }
}

void StockWidget::paintInterface(QPainter *p,
                                 const QStyleOptionGraphicsItem *option,
                                 const QRect &contentsRect)
{
    Q_UNUSED(option);
}

void StockWidget::addStock(const QString &source, const QString &sName)
{
    kDebug() << "source =" << source << ", sName =" << sName;
   
    StockItem *pStock = new StockItem(this);
    TrendItem *pTrend = new TrendItem(this);

    pStock->setId(source);
    pStock->setName(sName);
    
    m_vecStocks.append(pStock);
    m_vecTrends.append(pTrend);
    int row = m_vecStocks.size();

    m_pGridLayout->addItem(pStock, row, 0);
    m_pGridLayout->addItem(pTrend, row, 1);

    m_pStocksEngine->connectSource(source, this);
}

void StockWidget::sltNewSource(const QString &source)
{
    kDebug() << "source =" << source;
}

void StockWidget::sltSourceRemoved(const QString &source)
{
    kDebug() << "source =" << source;
    
    for (int i=m_vecStocks.size(); i>=0; i--)
    {
        if (m_vecStocks[i]->id() == source)
        {
            m_vecStocks.remove(i);
            m_vecTrends.remove(i);
        }
    }
}

void StockWidget::dataUpdated(const QString &source, Plasma::DataEngine::Data data)
{
    kDebug() << "source =" << source;
    
    for (int i=0; i<m_vecStocks.size(); i++)
    {
        if (m_vecStocks[i]->id() == source)
        {
            kDebug() << "Updating Value =" << data["Value"].toDouble();
            m_vecStocks[i]->setValue(data["Value"].toDouble());
            m_vecTrends[i]->setChange(data["Change"].toDouble());
            m_vecTrends[i]->setPercentChange(data["PercentChange"].toDouble());
        }
    }
    update();
}


#include "stockwidget.moc"

