/***************************************************************************
*   Copyright 2007 by Aaron Seigo <aseigo@kde.org>                        *
*   Copyright 2007 by Riccardo Iaconelli <riccardo@kde.org>               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
***************************************************************************/

#ifndef CLOCK_H
#define CLOCK_H

#include <QPixmap>
#include <QTimer>
#include <QPainter>
#include <QTime>
#include <QGraphicsItem>

#include <Plasma/Containment>
#include <Plasma/DataEngine>

#include <plasmaclock/clockapplet.h>
#include "ui_clockConfig.h"

class QTimer;

namespace Plasma
{
    class FrameSvg;
    class Svg;
    class SvgWidget;
    class Dialog;
}

class Clock : public ClockApplet
{
    Q_OBJECT
    public:
        Clock(QObject *parent, const QVariantList &args);
        ~Clock();
        
        void init();
        void constraintsEvent(Plasma::Constraints constraints);
//         QPainterPath shape() const;
        void paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *option, const QRect &contentsRect);
        
    public slots:
        void dataUpdated(const QString &name, const Plasma::DataEngine::Data &data);
        
    protected:
        void createClockConfigurationInterface(KConfigDialog *parent);
        void changeEngineTimezone(const QString &oldTimezone, const QString &newTimezone);
        
    protected slots:
        void clockConfigAccepted();
        void repaintNeeded();
        void animationSlot(int step);
        void finishAnimation();
        void adjustElements();
        
    private:
        void initElements();
        void connectToEngine();
        QString timeString();
        QRect tzRect();
        Plasma::FrameSvg *tzFrame();
        int getOffsetForDigit(int digitNumber);
        void animateUpdate();
        void updateSizes();
        
        bool m_showSecondHand;
        bool m_fancyHands;
        bool m_showTimezoneString;
        bool m_showingTimezone;
        Plasma::FrameSvg *m_tzFrame;
        Plasma::Svg *m_theme;
        QTime m_time;
        QTime m_lastTimeSeen;
        enum RepaintCache {
            RepaintNone,
            RepaintAll,
            RepaintHands
        };
        RepaintCache m_repaintCache;
        QPixmap m_faceCache;
        QPixmap m_handsCache;
        QPixmap m_glassCache;
        qreal m_verticalTranslation;
        QTimer *m_secondHandUpdateTimer;
        bool m_animateSeconds;
        int m_animationStart;
        /// Designer Config file
        Ui::clockConfig ui;
        
//         QTime m_time;
        QDate m_date;
        bool m_animating;
        
        bool m_showDate;
        bool m_showYear;
        bool m_showDay;
        bool m_showTimezone;
        bool m_twentyFour;
        QSize m_defaultElementSize;
        // Runtime sizing information
        int m_elWidth;
        int m_elHeight;
        int m_colonWidth;
        //int m_horizontalSpacing;
        //int m_verticalSpacing;
        int m_maxHeight;
        int m_maxWidth;
        int m_margin;
        QRect m_timeRect;
        QRect m_dateRect;
        QString m_prettyTimezone;
        QSizeF m_contentSize;
        QColor m_fgColor;
        int m_animationStep;
        
        // Default spacings
        int m_horizontalSpacing;
        int m_verticalSpacing;
        
        Plasma::SvgWidget *m_animatingWidget;
        Plasma::SvgWidget *m_animatingWidget2;
        
};

K_EXPORT_PLASMA_APPLET(train_clock, Clock)

#endif


#if 0
#ifndef CLOCK_H
#define CLOCK_H

#include <QTime>
#include <QDate>

#include <plasma/applet.h>
#include <plasma/dataengine.h>
#include <plasma/dialog.h>
#include "ui_clockConfig.h"
#include "plasmaclock/calendar.h"
#include <kdatepicker.h>



namespace Plasma
{
    class Svg;
}

class Clock : public Plasma::Applet
{
    Q_OBJECT
    public:
        Clock(QObject *parent, const QVariantList &args);
        ~Clock();

        void init();
        void paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *option, const QRect &contentsRect);
        void setPath(const QString&);
        void mousePressEvent(QGraphicsSceneMouseEvent *event);

        // reimplemented
        Qt::Orientations expandingDirections() const;

    public slots:
        void dataUpdated(const QString &name, const Plasma::DataEngine::Data &data);

    protected slots:
        void createConfigurationInterface(KConfigDialog *parent);
        void configAccepted();
        void showCalendar(QGraphicsSceneMouseEvent *event);
        void animationSlot(int step);

    protected:
        void constraintsEvent(Plasma::Constraints constraints);

    private:
        bool m_showDate;
        bool m_showYear;
        bool m_showDay;
        bool m_showTimezone;
        bool m_twentyFour;

        void animateUpdate();
        void updateSizes();
        int getOffsetForDigit(int digitNumber);

        QSize m_defaultElementSize;
        // Runtime sizing information
        int m_elWidth;
        int m_elHeight;
        int m_colonWidth;
        //int m_horizontalSpacing;
        //int m_verticalSpacing;
        int m_maxHeight;
        int m_maxWidth;
        int m_margin;
        QRect m_timeRect;
        QRect m_dateRect;

        QSizeF m_sizeHint;
        int m_pixelSize;
        QString m_timezone;
        Plasma::Svg* m_theme;
        QTime m_time;
        QDate m_date;
        Plasma::Dialog *m_calendar;
        QVBoxLayout *m_layout;
        QTime m_lastTimeSeen;
        QString m_prettyTimezone;
        QSizeF m_contentSize;
        QColor m_fgColor;

        // Designer Config file
        Ui::clockConfig ui;
//         Ui::calendar m_calendarUi;
        bool m_animating;
        int m_animationStep;

        // Default spacings
        int m_horizontalSpacing;
        int m_verticalSpacing;
};

K_EXPORT_PLASMA_APPLET(train_clock, Clock)

#endif
#endif