#include "thumbnailicon.h"

#include <QPixmap>
#include <QIcon>

// DEBUG ONLY
//#include <QPainter>

#include <KDebug>

#include "kwebthumbnailer.h"
#include "webbyeffect.h"
#include "reflectioneffect.h"

#define BLUR_RADIUS (5)

ThumbnailIcon::ThumbnailIcon( QGraphicsWidget *parent )
    : Plasma::IconWidget(parent)
{
    thumbnailer = new KWebThumbnailer(this);
    thumbnailer->setSize( QSize(96, 96) );
    connect( thumbnailer, SIGNAL(done(bool)), SLOT(thumbnailFinished(bool)) );
}

void ThumbnailIcon::setUrl( const QUrl &url )
{
    thumbnailer->setUrl( url );
}

void ThumbnailIcon::start()
{
    thumbnailer->start();
}

void ThumbnailIcon::thumbnailFinished( bool ok )
{
    if (!ok) {
        kDebug() << "Unable to load thumbnail";
        return;
    }

    QImage thumbImage = thumbnailer->thumbnail();
    QIcon thumb( QPixmap::fromImage(thumbImage) );
    setIcon( thumb );

#if 1
    ReflectionEffect *effect = new ReflectionEffect(this);
    setGraphicsEffect(effect);
#else
    WebbyEffect *effect = new WebbyEffect(this);
    effect->setReflectionImage(thumbImage);
    effect->setBlurRadius(BLUR_RADIUS);
    effect->setXOffset(3);

    QSizeF sz( size() );
    qreal yPos = sz.height()/2.0 + thumbImage.height()/2.0 + BLUR_RADIUS;
    qreal xPos = sz.width()/2.0 - thumbImage.width()/2.0 + 1;
    effect->setReflectionPosition( QPointF(xPos,yPos) );
    setGraphicsEffect(effect);
#endif
}

#if 0
// DEBUG ONLY
void ThumbnailIcon::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    IconWidget::paint(painter, option, widget);
    QGraphicsEffect *effect = graphicsEffect();
    if (!effect)
        return;

    painter->drawRect( effect->boundingRectFor(boundingRect()) );
}
#endif
