/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "contactsengine.h"

#include "Plasma/DataContainer"

#include "kabcsource.h"
#include "kopetesource.h"

ContactsEngine::ContactsEngine(QObject* parent, const QVariantList& args) :
	Plasma::DataEngine(parent)
{
	Q_UNUSED(args);

	setMinimumPollingInterval(1000);
	//setPollingInterval(2000);

	KABCSource *kabcSource = new KABCSource();
	connect(kabcSource, SIGNAL(newContact(Contact::Ptr)), this, SLOT(newContact(Contact::Ptr)));
	connect(kabcSource, SIGNAL(contactRemoved(Contact::Ptr)), this, SLOT(contactRemoved(Contact::Ptr)));
    connect(kabcSource, SIGNAL(contactChanged(Contact::Ptr)), this, SLOT(contactChanged(Contact::Ptr)));	
	kabcSource->initialize();

	KopeteSource *kopeteSource = new KopeteSource();
	connect(kopeteSource, SIGNAL(newContact(Contact::Ptr)), this, SLOT(newContact(Contact::Ptr)));
	connect(kopeteSource, SIGNAL(contactRemoved(Contact::Ptr)), this, SLOT(contactRemoved(Contact::Ptr)));
    connect(kopeteSource, SIGNAL(contactChanged(Contact::Ptr)), this, SLOT(contactChanged(Contact::Ptr)));    
	kopeteSource->initialize();
}

ContactsEngine::~ContactsEngine()
{
	// nothing to do
}

QStringList ContactsEngine::sources() const
{
	/** @todo add a "properties" and/or "contacts" source here, see nowplaying */

	return QStringList() << m_contacts.keys();
}

bool ContactsEngine::sourceRequestEvent(const QString &name)
{
	/** @todo sort out sources like "properties" here */

	return updateSourceEvent(name);
}

bool ContactsEngine::updateSourceEvent(const QString &name)
{
	if (m_contacts.contains(name))
	{
		Contact::Ptr contact = m_contacts[name];
		setData(name, "name", contact->displayName());
		setData(name, "photo", contact->photo());
		setData(name, "birthday", contact->birthday());
        setData(name, "instant_messaging_id", contact->instantMessagingId());
		setData(name, "instant_messaging_status", contact->onlineStatus());
		setData(name, "instant_messaging_status_message", contact->onlineStatusMessage());
        setData(name, "instant_messaging_unread_messages", contact->unreadInstantMessages());
		setData(name, "email", contact->email());
		setData(name, "location_known", contact->hasLocation());
		setData(name, "longitude", contact->location().x());
		setData(name, "latitude", contact->location().y());
		/** @todo: add more properties here */
	}
	else
	{
		// The source might be valid, but not available yet
		setData(name, DataEngine::Data());
	}

	return true;
}

void ContactsEngine::newContact(Contact::Ptr contact)
{
	m_contacts[contact->id()] = contact;
    updateSourceEvent(contact->id());
}

void ContactsEngine::contactRemoved(Contact::Ptr contact)
{
	m_contacts.remove(contact->id());
	updateSourceEvent(contact->id());
}

void ContactsEngine::contactChanged(Contact::Ptr contact)
{
    updateSourceEvent(contact->id());    
}

K_EXPORT_PLASMA_DATAENGINE(contacts, ContactsEngine)

#include "contactsengine.moc"
