/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "kopetecontact.h"

#include "kopete_interface.h"
#include "kopetesource.h"

#include <kurl.h>

#include <QtDBus/QDBusConnectionInterface>
#include <QtCore/QString>
#include <QtCore/QTimer>

KopeteContact::KopeteContact(const QString &id) :
	Contact(id)
{
    OrgKdeKopeteInterface *kopete_interface = new OrgKdeKopeteInterface(KOPETE_DBUS_SERVICE,
            KOPETE_DBUS_PATH, QDBusConnection::sessionBus());
    connect(kopete_interface, SIGNAL(contactChanged(QString)), this, SLOT(update(QString)));
    update(id);
}

QString KopeteContact::displayName()
{
	return m_displayName;
}

QString KopeteContact::photo()
{
	return m_photo;
}

QString KopeteContact::onlineStatus()
{
	return m_onlineStatus;
}

void KopeteContact::update(const QString &contactId)
{
    if (contactId != id())
    {
        return;
    }
    
	if (QDBusConnection::sessionBus().interface()->isServiceRegistered(KOPETE_DBUS_SERVICE))
	{
			OrgKdeKopeteInterface kopete_interface(KOPETE_DBUS_SERVICE,
					KOPETE_DBUS_PATH, QDBusConnection::sessionBus());

			QVariantMap properties = kopete_interface.contactProperties(contactId);
			m_instantMessagingId = properties["id"].toString();
			m_displayName = properties["display_name"].toString();
			m_onlineStatus = properties["status"].toString();
			KUrl photoUrl = properties["picture"].toString(); 
		    m_photo = photoUrl.toLocalFile();
		    m_statusMessage = properties["status_message"].toString();
		    m_unreadMessages = properties["pending_messages"].toStringList();
		    emit contactChanged(this);
	}
}

QString KopeteContact::onlineStatusMessage()
{
    return m_statusMessage;
}

QStringList KopeteContact::unreadInstantMessages()
{
    return m_unreadMessages;
}

QString KopeteContact::instantMessagingId()
{
    return m_instantMessagingId;
}

#include "kopetecontact.moc"

