/*
 *   Copyright (C) 2010 Ivan Cukic <ivan.cukic@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "genericlist.h"

#include <QFile>

#include <KStandardDirs>

// GenericListEngine

GenericListEngine::GenericListEngine(QObject * parent, const QVariantList & args)
    : Plasma::DataEngine(parent, args)
{
}

GenericListEngine::~GenericListEngine()
{
}

Plasma::Service * GenericListEngine::serviceForSource(const QString & name)
{
    if (name == ".meta") {
        return new MetaService();
    }
    return NULL;
}

bool GenericListEngine::sourceRequestEvent(const QString & name)
{
    return true;
}

bool GenericListEngine::updateSourceEvent(const QString & source)
{
    return true;
}

// MetaService

MetaService::MetaService(QObject * parent, const QVariantList & args)
    : Plasma::Service(parent, args)
{
}

void MetaService::registerOperationsScheme()
{
    if (name().isEmpty()) {
        kDebug() << "No name found";
        return;
    }

    QString path = KStandardDirs::locate("data", "plasma/services/" + name() + "-metaservice.operations");

    if (path.isEmpty()) {
        kDebug() << "Cannot find operations description:" << name() << "-metaservice.operations";
        return;
    }

    QFile file(path);
    setOperationsScheme(&file);
}

Plasma::ServiceJob * MetaService::createJob(const QString & operation, QMap < QString, QVariant > & parameters)
{
    return new MetaServiceJob(operation, parameters);
}

// MetaServiceJob

MetaServiceJob::MetaServiceJob(const QString & operation,
        const QMap < QString, QVariant > & parameters, QObject * parent)
    : Plasma::ServiceJob(".meta", operation, parameters, parent)
{

}

void MetaServiceJob::start()
{
    setResult("directory " + parameters()["directory"].toString());
}
