/*
 *   Copyright (C) 2008 Teemu Rytilahti <tpr@d5k.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2 (or, at
 *   your option, any later version) as published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef MYTHTVENGINE_H
#define MYTHTVENGINE_H

#include <plasma/dataengine.h>

#include <kio/job.h>

class KUrl;
class QByteArray;
class QDomNode;

class MythTVEngine : public Plasma::DataEngine
{
    Q_OBJECT

public:
    MythTVEngine(QObject* parent, const QVariantList& args);
    ~MythTVEngine();

protected:
    bool sourceRequestEvent(const QString &name);
    bool updateSourceEvent(const QString& source);

private:
    void parseEncoders(const QDomNode &node);
    void parseScheduled(const QDomNode &node);
    void parseQueue(const QDomNode &node);
    void parseMachineInfo(const QDomNode &node);
    
private:
    KUrl m_url;
    KIO::Job *m_transferJob;
    QByteArray m_data;
    
private Q_SLOTS:
    void dataFetched(KIO::Job *job, const QByteArray &data);
    void fetchingDone(KJob *);
};


K_EXPORT_PLASMA_DATAENGINE(mythtv, MythTVEngine)

#endif
