/* Copyright (C) 2009 Jon de Andres Frias <jondeandres@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) version 3, or any
 * later version accepted by the membership of KDE e.V. (or its
 * successor approved by the membership of KDE e.V.), which shall
 * act as a proxy defined in Section 6 of version 3 of the license.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "konqhistory.h"
#include <kdebug.h>



KonqHistoryRunner::KonqHistoryRunner(QObject *parent, const QVariantList &args) : Plasma::AbstractRunner(parent, args)
{
    Q_UNUSED(args)

    setObjectName("KDE History");
    m_historyProvider = new KonqHistoryProvider(this);
    m_historyProvider->loadHistory();
    m_icon = KIcon("view-history");

    m_historyList = m_historyProvider->entries();
}

// TODO: connect signals to update m_history
// TODO: get favicon using lib/konq/favicons/favicon.h

void KonqHistoryRunner::match(Plasma::RunnerContext &context)
{
    QList<Plasma::QueryMatch> matches;
    const QString term = context.query();
    if (!context.isValid()) {
       return;
    }
    qreal relevance = 0;

    foreach(const KonqHistoryEntry &entry, m_historyList) {

        Plasma::QueryMatch::Type type = Plasma::QueryMatch::NoMatch;
        KUrl url(entry.url);
        QString title = entry.title;

        if(url.isLocalFile())
            continue;

        //equal        
        if (title.compare(term, Qt::CaseInsensitive) == 0){
            type = Plasma::QueryMatch::ExactMatch;
            relevance = 1.0;

        } else if (url.host().compare(term, Qt::CaseInsensitive) == 0) {
            type = Plasma::QueryMatch::PossibleMatch;
            relevance = 1.0;
            
        //startsWitdh
        } else if (url.host().startsWith(term, Qt::CaseInsensitive) == 1) {
            type = Plasma::QueryMatch::PossibleMatch;
            relevance = 0.95;

        } else if (url.prettyUrl().startsWith(term, Qt::CaseInsensitive) == 1) {
            type = Plasma::QueryMatch::PossibleMatch;
            relevance = 0.95;
            
        } else if (title.startsWith(term, Qt::CaseInsensitive) == 1) {
            type = Plasma::QueryMatch::PossibleMatch;
            relevance = 0.8;

        //contains
        } else if (title.contains(term, Qt::CaseInsensitive)) {
            type = Plasma::QueryMatch::PossibleMatch;
            relevance = 0.4;

        } else if (url.prettyUrl().contains(term, Qt::CaseInsensitive)) {
            relevance = 0.4;
            type = Plasma::QueryMatch::PossibleMatch;
        }

        if(url.path(KUrl::RemoveTrailingSlash) != QString("/"))
            relevance -= 0.05;
            
        if (type != Plasma::QueryMatch::NoMatch) {
            Plasma::QueryMatch match(this);
            match.setType(type);
            match.setRelevance(relevance);
            match.setText(entry.title);
            match.setSubtext(entry.url.prettyUrl());
            match.setData(entry.url.url());
            match.setIcon(m_icon); //actually same as browserhistory runner
            matches << match;

        }
    }

    context.addMatches(term, matches);
}

void KonqHistoryRunner::run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &action)
{
    Q_UNUSED(context);
    KUrl url = (KUrl)action.data().toString();
    KToolInvocation::invokeBrowser(url.url());
}
