/*
 *   Copyright (C) 2008 Ryan P. Bitanga <ryan.bitanga@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <QAction>

#include <KDebug>
#include <KIcon>

//#include <Plasma/QueryAction>

#include "mediaplayer.h"

//This runner needs amarok 2 to work properly
MediaPlayerRunner::MediaPlayerRunner(QObject *parent, const QVariantList& args)
    : Plasma::AbstractRunner(parent, args),
      m_interface("org.kde.amarok", "/Player", "org.freedesktop.MediaPlayer")
{
    Q_UNUSED(args);

    setObjectName("Media Player Runner");
    setSpeed(AbstractRunner::SlowSpeed);
}

MediaPlayerRunner::~MediaPlayerRunner()
{
}

void MediaPlayerRunner::registerActions()
{
    QAction *play = addAction("media-play", KIcon("media-playback-start"), i18n("Play"));
    QAction *stop = addAction("media-stop", KIcon("media-playback-stop"), i18n("Stop"));
    QAction *next = addAction("media-next", KIcon("media-skip-forward"), i18n("Next Track"));
    QAction *prev = addAction("media-prev", KIcon("media-skip-backward"), i18n("Previous Track"));

    connect(play, SIGNAL(triggered(bool)), this, SLOT(play()));
    connect(stop, SIGNAL(triggered(bool)), this, SLOT(stop()));
    connect(prev, SIGNAL(triggered(bool)), this, SLOT(prev()));
    connect(next, SIGNAL(triggered(bool)), this, SLOT(next()));
}

void MediaPlayerRunner::match(Plasma::RunnerContext &context)
{
    if (context.query() != "player") {
        return;
    }

    Plasma::QueryMatch match(this);
    match.setType(Plasma::QueryMatch::ExactMatch);
    match.setIcon(KIcon("preferences-desktop-sound"));
    //Was amarok but this makes it easier to identify
    match.setText(i18n("Media Player"));
    context.addMatch(context.query(), match);
}

void MediaPlayerRunner::run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &match)
{
    Q_UNUSED(context)

    if (match.selectedAction()) {
        match.selectedAction()->trigger();
    } else {
        play();
    }
}

QList<QAction*> MediaPlayerRunner::actionsForMatch(const Plasma::QueryMatch &match)
{
    QList<QAction*> ret;
    //Check if actions were already loaded
    if (!action("media-play")) {
        registerActions();
    }

    ret << action("media-play") << action("media-stop")
        << action("media-prev") << action("media-next");

    return ret;
}

void MediaPlayerRunner::play()
{
    m_interface.call("Play");
}

void MediaPlayerRunner::stop()
{
    m_interface.call("Stop");
}

void MediaPlayerRunner::prev()
{
    m_interface.call("Prev");
}

void MediaPlayerRunner::next()
{
    m_interface.call("Next");
}

#include "mediaplayer.moc"
