/*
 *   Copyright 2008 Aike J Sommer <dev@aikesommer.name>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


#ifndef SCREENSAPPLET_H
#define SCREENSAPPLET_H

#include <QObject>
#include <QMap>

#include <Plasma/Applet>
#include <Plasma/DataEngine>


class QGraphicsLinearLayout;

class ScreenWidget;
class ScreensWidget;
class OutputHandler;

namespace Plasma {
    class PushButton;
    class Meter;
}


class ScreensApplet : public Plasma::Applet
{
    Q_OBJECT
    public:
        ScreensApplet(QObject *parent, const QVariantList &args);

        void paintInterface(QPainter *painter,
                const QStyleOptionGraphicsItem *option,
                const QRect& contentsRect);
        void init();
        
    public slots:
        void dataUpdated(const QString & sourceName, const Plasma::DataEngine::Data & data);
        void sourceAdded(const QString & sourceName);
        void sourceRemoved(const QString & sourceName);
        
        void resizeRequested(QString outputId, QSize size);
        void moveRequested(QString outputId, QPoint position);
        
        void confirm();
        void revert();

    private:
        void buildOutputs();

        //KIcon m_icon;
        QSize m_totalSize;
        
        QMap<QString, Plasma::DataEngine::Data> m_screenData;
        QMap<QString, ScreenWidget *> m_screens;
        QMap<QString, OutputHandler *> m_outputHandlers;
        ScreensWidget * m_screensContainer;

        int m_confirmLeft;
        int m_confirmMax;
        
        Plasma::PushButton * m_revertButton;
        Plasma::PushButton * m_confirmButton;
        Plasma::Meter * m_confirmMeter;
        
        QGraphicsLinearLayout * m_layout;
        QGraphicsLinearLayout * m_statusLayout;
};

class OutputHandler : public QObject {
    Q_OBJECT
    
    public:
        OutputHandler(QObject * parent, QString outputId);
            
    public Q_SLOTS:
        void resizeRequested(QSize size);
        void moveRequested(QPoint position);
        
    Q_SIGNALS:
        void resizeRequested(QString outputId, QSize size);
        void moveRequested(QString outputId, QPoint position);
        
    private:
        QString m_outputId;
};


#endif // SCREENSAPPLET_H

