/*
 *   Copyright (C) 2008 Petri Damsten <damu@iki.fi>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "krossdataenginescript.h"
#include <plasma/dataengine.h>

KrossDataEngineScript::KrossDataEngineScript(QObject* parent,
                                             const QVariantList &args,
                                             const QString &language)
    : Plasma::DataEngineScript(parent), m_language(language)
{
    Q_UNUSED(args)
}

KrossDataEngineScript::~KrossDataEngineScript()
{
}

void KrossDataEngineScript::setData(const QString& source, const QString& key,
                                    const QVariant& value)
{
    DataEngineScript::setData(source, key, value);
}

void KrossDataEngineScript::setData(const QString &source, const QVariant &value)
{
    DataEngineScript::setData(source, value);
}

void KrossDataEngineScript::removeAllData(const QString& source)
{
    DataEngineScript::removeAllData(source);
}

void KrossDataEngineScript::removeData(const QString& source, const QString& key)
{
    DataEngineScript::removeData(source, key);
}

void KrossDataEngineScript::setMinimumPollingInterval(int minimumMs)
{
    DataEngineScript::setMinimumPollingInterval(minimumMs);
}

int KrossDataEngineScript::minimumPollingInterval() const
{
    return DataEngineScript::minimumPollingInterval();
}

void KrossDataEngineScript::setPollingInterval(uint frequency)
{
    DataEngineScript::setPollingInterval(frequency);
}

void KrossDataEngineScript::removeAllSources()
{
    DataEngineScript::removeAllSources();
}

bool KrossDataEngineScript::init()
{
    QString name;
    if (dataEngine() && !dataEngine()->name().isEmpty()) {
        name = dataEngine()->name();
        if (initScript(this, m_language, name, "dataengine")) {
            callFunction("initScript");
            return true;
        }
    }
    kDebug() << "Could not init script" << m_language << name;
    return false;
}

bool KrossDataEngineScript::sourceRequestEvent(const QString &name)
{
    return callFunction("sourceRequestedEvent", QVariantList() << name).toBool();
}

bool KrossDataEngineScript::updateSourceEvent(const QString& source)
{
    return callFunction("updateSourceEvent", QVariantList() << source).toBool();
}

#include "krossdataenginescript.moc"
