/*
 *   Copyright (C) 2008 Petri Damsten <damu@iki.fi>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "searchmatchscript.h"
#include <Plasma/QueryMatch>
#include <KIcon>
#include <QVariant>

SearchMatchScript::SearchMatchScript(Plasma::AbstractRunner *runner)
    : m_queryMatch(runner), m_constQueryMatch(m_queryMatch)
{
}

SearchMatchScript::SearchMatchScript(const Plasma::QueryMatch& match)
    : m_queryMatch(0), m_constQueryMatch(match)
{
}

void SearchMatchScript::setType(int type)
{
    m_queryMatch.setType((Plasma::QueryMatch::Type)type);
}

void SearchMatchScript::setRelevance(qreal relevance)
{
    m_queryMatch.setRelevance(relevance);
}

void SearchMatchScript::setId(const QString& id)
{
    m_queryMatch.setId(id);
}

void SearchMatchScript::setIcon(const QString& icon)
{
    m_queryMatch.setIcon(KIcon(icon));
}

void SearchMatchScript::setText(const QString& text)
{
    m_queryMatch.setText(text);
}

void SearchMatchScript::setSubtext(const QString& text)
{
    m_queryMatch.setSubtext(text);
}

void SearchMatchScript::setData(const QVariant& data)
{
    m_queryMatch.setData(data);
}

void SearchMatchScript::setEnabled(bool enable)
{
    m_queryMatch.setEnabled(enable);
}

int SearchMatchScript::type()
{
    return m_constQueryMatch.type();
}

qreal SearchMatchScript::relevance()
{
    return m_constQueryMatch.relevance();
}

QString SearchMatchScript::id()
{
    return m_constQueryMatch.id();
}

QString SearchMatchScript::text()
{
    return m_constQueryMatch.text();
}

QString SearchMatchScript::subtext()
{
    return m_constQueryMatch.subtext();
}

QVariant SearchMatchScript::data()
{
    return m_constQueryMatch.data();
}

bool SearchMatchScript::isEnabled()
{
    return m_constQueryMatch.isEnabled();
}

#include "searchmatchscript.moc"
