/*
 *   Copyright (C) 2008 Petri Damsten <damu@iki.fi>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef SEARCH_MATCH_SCRIPT_H
#define SEARCH_MATCH_SCRIPT_H

#include <Plasma/QueryMatch>
#include <QObject>
#include <QString>
#include <QMetaType>

namespace Plasma {
    class AbstractRunner;
}

class SearchMatchScript : public QObject
{
    Q_OBJECT
    Q_ENUMS(MatchType)
public:
    // From Plasma::QueryMatch
    enum MatchType { NoMatch = 0, CompletionMatch = 10,
                     PossibleMatch = 30, InformationalMatch = 50,
                     HelperMatch = 70, ExactMatch = 100 };

    SearchMatchScript(Plasma::AbstractRunner *runner);
    SearchMatchScript(const Plasma::QueryMatch& match);
    ~SearchMatchScript() {};

    Plasma::QueryMatch& match() { return m_queryMatch; };

public slots:
    void setRelevance(qreal relevance);
    void setId(const QString &id);
    void setType(int type);
    void setIcon(const QString& icon);
    void setText(const QString& text);
    void setSubtext(const QString& text);
    void setData(const QVariant& data);
    void setEnabled(bool enable);
    int type();
    qreal relevance();
    QString id();
    QString text();
    QString subtext();
    QVariant data();
    bool isEnabled();

private:
    Plasma::QueryMatch m_queryMatch;
    const Plasma::QueryMatch& m_constQueryMatch;
};

#endif // SEARCH_MATCH_SCRIPT_H

