/***************************************************************************
 *   Copyright (C) 2009 by Wang Hoi <zealot.hoi@gmail.com>                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "koratminibar.h"
#include "koratbutton.h"
#include "koratdefines.h"

#include <plasma/framesvg.h>

#include <KDebug>

#include <QGraphicsLinearLayout>

namespace Korat
{
class MiniBarPrivate
{
public:
    MiniBarPrivate(MiniBar *bar)
        : q(bar)
        , background(new Plasma::FrameSvg())
        , layout(new QGraphicsLinearLayout())
    {
    }

    MiniBar *q;

    Plasma::FrameSvg *background;

    QGraphicsLinearLayout *layout;

    QList<Item *> itemList;
};

MiniBar::MiniBar(QGraphicsItem *parent)
    : Item(parent)
    , d(new MiniBarPrivate(this))
{
    //setPreferredHeight(LARGEST_HEIGHT/3.0);
    setMaximumHeight(LARGEST_HEIGHT/3.0);

    d->background->setImagePath("widgets/frame");
    d->background->setElementPrefix("raised");

    qreal left,top,right,bottom;
    d->background->getMargins(left,top,right,bottom);
    d->layout->setSpacing(0);
    d->layout->setContentsMargins(left,top,right,bottom);
    setLayout(d->layout);
}

MiniBar::~MiniBar()
{
    delete d;
    d = 0;
}

bool MiniBar::isContainer()
{
    return true;
}

void MiniBar::addItem(Item *item)
{
    d->itemList << item;
    d->layout->addItem(item);
    emit sizeChanged();
}

void MiniBar::removeItem(int index)
{
    if ((index >= 0) && (index < d->itemList.size())) {
        d->itemList.removeAt(index);
        d->layout->removeAt(index);
        emit sizeChanged();
    }
}

void MiniBar::removeItem(Item *item)
{
    removeItem(d->itemList.indexOf(item));
}

Item *MiniBar::getItem(int index)
{
    return d->itemList.at(index);
}

void MiniBar::resizeEvent(QGraphicsSceneResizeEvent * event)
{
    Item::resizeEvent(event);
    d->background->resizeFrame(event->newSize());
}

void MiniBar::paint(QPainter *painter,const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    Q_UNUSED(option);
    Q_UNUSED(widget);

    d->background->paintFrame(painter);
}

QList<WeightList> MiniBar::availableWidths()
{
    QList<WeightList> result;
    WeightList weightList;

    weightList << (int)ItemMiniBar;

    result << weightList;

    //kDebug() << result;
    return result;
}

qreal MiniBar::getWidth(const WeightList &weightList)
{
    qreal width = 0;

    foreach (Item *item, d->itemList) {
        QList<WeightList> itemWeightList = item->availableWidths();
        WeightList tempWeightList;
        tempWeightList.clear();
        foreach (const WeightList &wList, itemWeightList) {
            // always pickup the smallest width ( last() ).
            tempWeightList << wList.last();
        }
        width += item->getWidth(tempWeightList);
    }

    qreal left,top,right,bottom;
    d->background->getMargins(left,top,right,bottom);

    //kDebug() << weightList << width + left + right;
    return width + left + right;
}

void MiniBar::adjustWidth(const WeightList &weightList)
{
    qreal minWidth = getWidth(weightList);
    setMinimumWidth(minWidth);
    setMaximumWidth(minWidth);
    foreach (Item *item, d->itemList) {
        QList<WeightList> itemWeightList = item->availableWidths();
        WeightList tempWeightList;
        tempWeightList.clear();
        foreach (const WeightList &wList, itemWeightList) {
            // always pickup the smallest width ( last() ).
            tempWeightList << wList.last();
        }
        item->adjustWidth(tempWeightList);
    }
}

} // namespace Korat

#include "koratminibar.moc"

// vim: sw=4 sts=4 et tw=100
