#include "shadowitem.h"

#include "composition.h"
#include "layer.h"
#include "node.h"

#include <QPainter>
#include <QImage>

ShadowItem::ShadowItem()
{
    setZValue(3);
    m_shadowParent = 0;
}

void ShadowItem::setShadowParent(QGraphicsItem *item)
{
    m_shadowParent = item;
    adjustPosition();
    generate();
}


QGraphicsItem * ShadowItem::shadowParent() const
{
    return m_shadowParent;
}


void ShadowItem::generate()
{
    QPainterPath path = m_shadowParent->shape();
    QRectF rect = path.boundingRect();
    QSize s = rect.size().toSize();
    QImage img(s, QImage::Format_ARGB32_Premultiplied);
    img.fill(0);
    QPainter p(&img);
    p.setRenderHint(QPainter::Antialiasing);
    p.fillPath(path, Qt::gray);
    p.end();

    Quasar::Composition compo(this);
    Quasar::Node *filter = compo.createNode("/blur/exponential");
    filter->setPropertyValue("input", Quasar::Layer(img));
    Quasar::Layer out = compo.renderToLayer();
    img = out.toImage();
    setPixmap(QPixmap::fromImage(img));
}


void ShadowItem::adjustPosition()
{
    setPos(m_shadowParent->pos() - QPointF(12, 12));
}
