#define GL_GLEXT_PROTOTYPES
#include "edgedetection_p.h"


#include <QVariant>
#include <QDebug>

#include <GL/glext.h>
#include <GL/gl.h>

using namespace Quasar;


static const char *vertShaderText =
    "void main() {\n"
    "   gl_FrontColor = gl_Color;\n"
    "   gl_TexCoord[0] = gl_MultiTexCoord0;\n"
    "   gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;\n"
    "}\n";

static const char *fragShaderText =
    "uniform sampler2D srcTex;\n"
    "uniform float DeltaX;\n"
    "uniform float DeltaY;\n"
    "uniform float Threshold;\n"
    "void main() {\n"
    "    vec2 ox = vec2(DeltaX, 0.0);\n"
    "    vec2 oy = vec2(0.0, DeltaY);\n"
    "    vec2 PP = gl_TexCoord[0].st - oy;\n"
    "    float g00 = texture2D(srcTex, PP - ox).x;\n"
    "    float g01 = texture2D(srcTex, PP).x;\n"
    "    float g02 = texture2D(srcTex, PP + ox).x;\n"
    "    PP = gl_TexCoord[0].st;\n"
    "    float g10 = texture2D(srcTex, PP - ox).x;\n"
    "    float g12 = texture2D(srcTex, PP + ox).x;\n"
    "    PP = gl_TexCoord[0].st + oy;\n"
    "    float g20 = texture2D(srcTex, PP - ox).x;\n"
    "    float g21 = texture2D(srcTex, PP).x;\n"
    "    float g22 = texture2D(srcTex, PP + ox).x;\n"
    "    float sx = g20 + g22 - g00 - g02 + 2 * (g21 - g01);\n"
    "    float sy = g22 + g02 - g00 - g20 + 2 * (g12 - g10);\n"
    "    float dist = (sx * sx + sy *sy);\n"
    "    float tSq = Threshold * Threshold;\n"
    "    float result = 1;\n"
    "    if (dist > tSq) { result = 0; }\n"
    "    gl_FragColor =  vec4(result);\n"
    "}\n";

EdgeDetectionNode::EdgeDetectionNode()
    : FilterNode("EdgeDetection")
{
    setShaderSources(vertShaderText, fragShaderText);
    Quasar::Property aprop = {
        Quasar::Property::InputProperty,

        QString::fromLatin1("Threshold"),
        QVariant::Double,
        1.5,
        QString::fromLatin1("Threshold"),

        1.5,
        0,
        2,
    };
    addProperty(QString::fromLatin1("Threshold"), aprop);
}

QRectF EdgeDetectionNode::domainOfDefinition(const QRectF &rect) const
{
    return rect;
}

QRectF EdgeDetectionNode::regionOfInterest(const QRectF &rect) const
{
    return rect;
}

void EdgeDetectionNode::aboutToExecute()
{
}

void EdgeDetectionNode::programChanged(GLuint prog)
{
    GLuint texLoc = glGetUniformLocationARB(prog, "srcTex");
    glUniform1iARB(texLoc, 0);
    QVariant var = propertyValue("Threshold");
    float res = var.toDouble();
    setShaderUniform("Threshold", res);
}

void EdgeDetectionNode::propertyChanged(const QString &name,
                                        const QVariant &val)
{
    FilterNode::propertyChanged(name, val);
    if (name == "texture") {
        glUseProgram(programId());
        Texture tex = qvariant_cast<Texture>(val);
        setShaderUniform("DeltaX", (float)(1.0/tex.width()));
        setShaderUniform("DeltaY", (float)(1.0/tex.height()));
        glUseProgram(0);
    }
}
