#define GL_GLEXT_PROTOTYPES
#include "fadenode_p.h"

#include <QVariant>
#include <QDebug>

#include <GL/glext.h>
#include <GL/gl.h>

using namespace Quasar;


static const char *vertShaderText =
    "void main() {\n"
    "   gl_FrontColor = gl_Color;\n"
    "   gl_TexCoord[0] = gl_MultiTexCoord0;\n"
    "   gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;\n"
    "}\n";

static const char *fragShaderText =
    "uniform sampler2D srcTex;\n"
    "uniform float Alpha;\n"
    "void main() {\n"
    "    vec4 fragColor = texture2D(srcTex,  gl_TexCoord[0].st).rgba;\n"
    "    fragColor.a = fragColor.a * Alpha;\n"
    "    gl_FragColor =  fragColor;\n"
    "}\n";

FadeNode::FadeNode()
    : FilterNode("Fade")
{
    setShaderSources(vertShaderText, fragShaderText);
    Quasar::Property aprop = {
        Quasar::Property::InputProperty,

        QString::fromLatin1("Alpha"),
        QVariant::Double,
        1.0,
        QString::fromLatin1("Fade alpha"),

        1.0,
        0,
        1,
    };
    addProperty(QString::fromLatin1("Alpha"), aprop);
}

QRectF FadeNode::domainOfDefinition(const QRectF &rect) const
{
    return rect;
}

QRectF FadeNode::regionOfInterest(const QRectF &rect) const
{
    return rect;
}

void FadeNode::programChanged(GLuint prog)
{
    GLuint texLoc = glGetUniformLocationARB(prog, "srcTex");
    glUniform1iARB(texLoc, 0);
    QVariant var = propertyValue("alpha");
    float res = var.toDouble();
    m_alphaLoc =  glGetUniformLocationARB(prog, "Alpha");
    glUniform1fARB(m_alphaLoc, res);
}
